from __future__ import annotations

from enum import Enum

GIT_HISTORY_NOT_BEEN_REMOVED = "not-removed"


class ValidationStatus(Enum):
    PRIVILEGED = "Privileged"
    VALID = "Valid"
    INVALID = "Invalid"
    UNKNOWN = "Unknown"
    UNAVAILABLE = "Unavailable"

    def __str__(self) -> str:
        # needed, because of a Python 3.11 change
        return self.value


SECRET_VALIDATION_STATUSES = {
    ValidationStatus.VALID.value,
    ValidationStatus.PRIVILEGED.value,
    ValidationStatus.INVALID.value,
    ValidationStatus.UNKNOWN.value,
    ValidationStatus.UNAVAILABLE.value,
}


class VerifySecretsResult(Enum):
    INSUFFICIENT_PARAMS = "INSUFFICIENT_PARAMS"
    FAILURE = "FAILURE"
    SUCCESS = "SUCESS"

    def __str__(self) -> str:
        # needed, because of a Python 3.11 change
        return self.value



from __future__ import annotations

import itertools
import logging
import os
import re
from abc import ABC, abstractmethod
from collections.abc import Iterable
from typing import List, Any, TYPE_CHECKING, TypeVar, Generic, Dict, Optional

from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.util.tqdm_utils import ProgressBar

from checkov.common.graph.checks_infra.base_check import BaseGraphCheck
from checkov.common.output.report import Report
from checkov.runner_filter import RunnerFilter
from checkov.common.graph.graph_manager import GraphManager  # noqa

if TYPE_CHECKING:
    from checkov.common.checks_infra.registry import Registry
    from checkov.common.graph.checks_infra.registry import BaseRegistry
    from checkov.common.typing import _CheckResult, LibraryGraphConnector, LibraryGraph

_Context = TypeVar("_Context", bound="dict[Any, Any]|None")
_Definitions = TypeVar("_Definitions", bound="dict[Any, Any]|None")
_GraphManager = TypeVar("_GraphManager", bound="GraphManager[Any, Any]|None")


def strtobool(val: str) -> int:
    """Convert a string representation of truth to true (1) or false (0).

    True values are 'y', 'yes', 't', 'true', 'on', and '1'; false values
    are 'n', 'no', 'f', 'false', 'off', and '0'.  Raises ValueError if
    'val' is anything else.
    """
    val = val.lower()
    if val in ('y', 'yes', 't', 'true', 'on', '1'):
        return 1
    elif val in ('n', 'no', 'f', 'false', 'off', '0'):
        return 0
    else:
        raise ValueError("invalid boolean value %r for environment variable CKV_IGNORE_HIDDEN_DIRECTORIES" % (val,))


IGNORED_DIRECTORIES_ENV = os.getenv("CKV_IGNORED_DIRECTORIES", "node_modules,.terraform,.serverless")
IGNORE_HIDDEN_DIRECTORY_ENV = strtobool(os.getenv("CKV_IGNORE_HIDDEN_DIRECTORIES", "True"))

ignored_directories = IGNORED_DIRECTORIES_ENV.split(",")


class BaseRunner(ABC, Generic[_Definitions, _Context, _GraphManager]):
    check_type = ""
    definitions: _Definitions | None = None
    raw_definitions: dict[str, list[tuple[int, str]]] | None = None
    context: _Context | None = None
    breadcrumbs = None
    external_registries: list[BaseRegistry] | None = None
    graph_manager: _GraphManager | None = None
    graph_registry: Registry | None = None
    db_connector: LibraryGraphConnector
    resource_subgraph_map: Optional[dict[str, str]] = None

    def __init__(self, file_extensions: Iterable[str] | None = None, file_names: Iterable[str] | None = None):
        self.file_extensions = file_extensions or []
        self.file_names = file_names or []
        self.pbar = ProgressBar(self.check_type)
        db_connector_class: "type[NetworkxConnector | RustworkxConnector]" = NetworkxConnector
        graph_framework = os.getenv("CHECKOV_GRAPH_FRAMEWORK", "RUSTWORKX")
        if graph_framework == "RUSTWORKX":
            from checkov.common.graph.db_connectors.rustworkx.rustworkx_db_connector import RustworkxConnector
            db_connector_class = RustworkxConnector

        self.db_connector = db_connector_class()

    @abstractmethod
    def run(
            self,
            root_folder: str | None,
            external_checks_dir: list[str] | None = None,
            files: list[str] | None = None,
            runner_filter: RunnerFilter | None = None,
            collect_skip_comments: bool = True,
    ) -> Report | list[Report]:
        pass

    def should_scan_file(self, filename: str) -> bool:
        # runners that are always applicable can do nothing and be included
        if not self.file_extensions and not self.file_names:
            return True

        basename = os.path.basename(filename)
        if basename and self.file_names and basename in self.file_names:
            return True

        extension = os.path.splitext(filename)[1]
        if extension and self.file_extensions and extension in self.file_extensions:
            return True

        return False

    def included_paths(self) -> Iterable[str]:
        return []

    def set_external_data(
            self,
            definitions: _Definitions | None,
            context: _Context | None,
            breadcrumbs: dict[str, dict[str, Any]] | None,
            **kwargs: Any,
    ) -> None:
        self.definitions = definitions
        self.context = context
        self.breadcrumbs = breadcrumbs

    def set_raw_definitions(self, definitions_raw: dict[str, list[tuple[int, str]]] | None) -> None:
        self.definitions_raw = definitions_raw

    def populate_metadata_dict(self) -> None:
        return None

    def load_external_checks(self, external_checks_dir: List[str]) -> None:
        return None

    def get_graph_checks_report(self, root_folder: str, runner_filter: RunnerFilter) -> Report:
        return Report(check_type="not_defined")

    def run_graph_checks_results(self, runner_filter: RunnerFilter, report_type: str, graph: LibraryGraph | None = None
                                 ) -> dict[BaseGraphCheck, list[_CheckResult]]:
        checks_results: "dict[BaseGraphCheck, list[_CheckResult]]" = {}
        if graph is None and (not self.graph_manager or not self.graph_registry):
            # should not happen
            logging.warning("Graph components were not initialized")
            return checks_results

        if graph is None and isinstance(self.graph_manager, GraphManager):
            graph = self.graph_manager.get_reader_endpoint()
        for r in itertools.chain(self.external_registries or [], [self.graph_registry]):
            r.load_checks()  # type:ignore[union-attr]
            registry_results = r.run_checks(graph, runner_filter, report_type)  # type:ignore[union-attr]
            checks_results = {**checks_results, **registry_results}
        # Filtering the checks now
        filtered_result: Dict[BaseGraphCheck, List[_CheckResult]] = {}
        for check, results in checks_results.items():
            filtered_result[check] = [result for result in results if runner_filter.should_run_check(
                check,
                check_id=check.id,
                file_origin_paths=[result.get("entity", {}).get(CustomAttributes.FILE_PATH, "")],
                report_type=self.check_type
            )]

        return filtered_result


def filter_ignored_paths(
    root_dir: str,
    names: list[str] | list[os.DirEntry[str]],
    excluded_paths: list[str] | None,
    included_paths: Iterable[str] | None = None
) -> None:
    # we need to handle legacy logic, where directories to skip could be specified using the env var (default value above)
    # or a directory starting with '.'; these look only at directory basenames, not relative paths.
    #
    # But then any other excluded paths (specified via --skip-path or via the platform repo settings) should look at
    # the path name relative to the root folder. These can be files or directories.
    # Example: take the following dir tree:
    # .
    #   ./dir1
    #      ./dir1/dir33
    #      ./dir1/.terraform
    #   ./dir2
    #      ./dir2/dir33
    #      /.dir2/hello.yaml
    #
    # if excluded_paths = ['dir1/dir33', 'dir2/hello.yaml'], then we would scan dir1, but we would skip its subdirectories. We would scan
    # dir2 and its subdirectory, but we'd skip hello.yaml.

    # first handle the legacy logic - this will also remove files starting with '.' but that's probably fine
    # mostly this will just remove those problematic directories hardcoded above.
    included_paths = included_paths or []
    for entry in list(names):
        path = entry.name if isinstance(entry, os.DirEntry) else entry
        if path in ignored_directories:
            safe_remove(names, entry)
        if path.startswith(".") and IGNORE_HIDDEN_DIRECTORY_ENV and path not in included_paths:
            safe_remove(names, entry)

    # now apply the new logic
    # TODO this is not going to work well on Windows, because paths specified in the platform will use /, and
    #  paths specified via the CLI argument will presumably use \\
    if excluded_paths:
        compiled = []
        for p in excluded_paths:
            try:
                compiled.append(re.compile(p.replace(".terraform", r"\.terraform")))
            except re.error:
                # do not add compiled paths that aren't regexes
                continue
        for entry in list(names):
            path = entry.name if isinstance(entry, os.DirEntry) else entry
            full_path = os.path.join(root_dir, path)
            if any(pattern.search(full_path) for pattern in compiled) or any(p in full_path for p in excluded_paths):
                safe_remove(names, entry)


def safe_remove(names: list[Any], path: Any) -> None:
    if path in names:
        names.remove(path)



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ImageTagFixed(BaseResourceCheck):

    def __init__(self):
        """
         You should avoid using the :latest tag when deploying containers in production
         as it is harder to track which version of the image is running
         and more difficult to roll back properly.
         """
        name = "Image Tag should be fixed - not latest or blank"
        id = "CKV_K8S_14"
        supported_resources = ["kubernetes_pod", "kubernetes_pod_v1",
                               "kubernetes_deployment", "kubernetes_deployment_v1"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        spec = conf.get('spec', [None])[0]
        if isinstance(spec, dict) and spec:
            evaluated_keys_path = "spec"

            template = spec.get("template")
            if template and isinstance(template, list):
                template = template[0]
                template_spec = template.get("spec")
                if template_spec and isinstance(template_spec, list):
                    spec = template_spec[0]
                    evaluated_keys_path = f'{evaluated_keys_path}/[0]/template/[0]/spec'

            if spec.get("container"):
                containers = spec.get("container")
                for idx, container in enumerate(containers):
                    if not isinstance(container, dict):
                        return CheckResult.UNKNOWN
                    if container.get("image"):
                        name = container.get("image")[0]
                        if ":" in name:
                            if name.split(":")[1] in ("latest", ""):
                                self.evaluated_keys = [f'{evaluated_keys_path}/[0]/container/[{idx}]/image']
                                return CheckResult.FAILED
                            continue
                        if "@" in name:
                            continue
                        self.evaluated_keys = [f'{evaluated_keys_path}/[0]/container/[{idx}]/image']
                        return CheckResult.FAILED
                    self.evaluated_keys = [f'{evaluated_keys_path}/[0]/container/[{idx}]']
                    return CheckResult.FAILED
                return CheckResult.PASSED
            return CheckResult.FAILED


check = ImageTagFixed()



"""
======
update-checker
======

code from:
https://github.com/bboe/update_checker
package:
https://pypi.org/project/update-checker/

Public resources from ``update-checker``
"""

from checkov.common.util.update_checker.update_checker import UpdateChecker

__all__ = ("UpdateChecker",)



from __future__ import annotations

import logging
import os
from typing import TYPE_CHECKING, Any, Iterable

from checkov.circleci_pipelines.registry import registry
from checkov.common.output.report import CheckType, Report
from checkov.runner_filter import RunnerFilter
from checkov.yaml_doc.runner import Runner as YamlRunner

if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry

WORKFLOW_DIRECTORY = "circleci"


class Runner(YamlRunner):
    check_type = CheckType.CIRCLECI_PIPELINES  # noqa: CCE003  # a static attribute

    def require_external_checks(self) -> bool:
        return False

    def import_registry(self) -> BaseCheckRegistry:
        return registry

    def included_paths(self) -> list[str]:
        return [".circleci"]

    @staticmethod
    def _parse_file(
        f: str, file_content: str | None = None
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        if Runner.is_workflow_file(f):
            return YamlRunner._parse_file(f)

        return None

    @staticmethod
    def is_workflow_file(file_path: str) -> bool:
        """
        :return: True if the file mentioned is named config.yml/yaml in .circleci dir from included_paths(). Otherwise: False
        """
        abspath = os.path.abspath(file_path)
        return WORKFLOW_DIRECTORY in abspath and abspath.endswith(("config.yml", "config.yaml"))

    def get_resource(self, file_path: str, key: str, supported_entities: Iterable[str],
                     start_line: int = -1, end_line: int = -1, graph_resource: bool = False) -> str:
        """
        supported resources for circleCI:
            jobs.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}
            jobs.*.steps[]
            orbs.{orbs: @}
        """
        if len(list(supported_entities)) > 1:
            logging.debug("order of entities might cause extracting the wrong key for resource_id")
        new_key = key
        definition = self.definitions.get(file_path, {})
        if not definition or not isinstance(definition, dict):
            return new_key
        if 'orbs.{orbs: @}' in supported_entities:
            new_key = "orbs"
        elif 'jobs.*.steps[]' in supported_entities:
            job_name = self.resolve_sub_name(definition, start_line, end_line, tag='jobs')
            step_name = self.resolve_step_name(definition['jobs'].get(job_name), start_line, end_line)
            new_key = f'jobs({job_name}).steps{step_name}' if job_name else "jobs"
        elif 'jobs.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}' in supported_entities:
            job_name = self.resolve_sub_name(definition, start_line, end_line, tag='jobs')
            image_name = self.resolve_image_name(definition['jobs'].get(job_name), start_line, end_line)
            new_key = f'jobs({job_name}).docker.image{image_name}' if job_name else "jobs"
        elif 'executors.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}':
            executor_name = self.resolve_sub_name(definition, start_line, end_line, tag='executors')
            image_name = self.resolve_image_name(definition['executors'].get(executor_name), start_line, end_line)
            new_key = f'executors({executor_name}).docker.image{image_name}' if executor_name else "executors"
        return new_key

    def run(
            self,
            root_folder: str | None = None,
            external_checks_dir: list[str] | None = None,
            files: list[str] | None = None,
            runner_filter: RunnerFilter | None = None,
            collect_skip_comments: bool = True,
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        report = super().run(root_folder=root_folder, external_checks_dir=external_checks_dir,
                             files=files, runner_filter=runner_filter, collect_skip_comments=collect_skip_comments)
        return report



from __future__ import annotations

from typing import Dict, Any

from checkov.common.graph.checks_infra.base_check import BaseGraphCheck


class BaseGraphCheckParser:
    def validate_check_config(self, file_path: str, raw_check: dict[str, dict[str, Any]]) -> bool:
        """Validates the graph check config"""
        return True

    def parse_raw_check(self, raw_check: Dict[str, Dict[str, Any]], **kwargs: Any) -> BaseGraphCheck:
        raise NotImplementedError



from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories


class SagemakerDataQualityJobDefinitionVolumeEncryption(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Amazon Sagemaker Data Quality Job uses KMS to encrypt data on attached storage volume"
        id = "CKV_AWS_368"
        supported_resources = ["AWS::SageMaker::DataQualityJobDefinition"]
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'Properties/JobResources/ClusterConfig/VolumeKmsKeyId'

    def get_expected_value(self):
        return ANY_VALUE


check = SagemakerDataQualityJobDefinitionVolumeEncryption()



from typing import Any
from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class CognitiveServicesConfigureIdentity(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Cognitive Services account is not configured with managed identity"
        id = "CKV_AZURE_238"
        supported_resources = ('Microsoft.CognitiveServices/accounts',)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "identity/type"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = CognitiveServicesConfigureIdentity()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class DocDBGlobalClusterEncryption(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure DocumentDB Global Cluster is encrypted at rest (default is unencrypted)"
        id = "CKV_AWS_292"
        supported_resources = ('aws_docdb_global_cluster',)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "storage_encrypted"


check = DocDBGlobalClusterEncryption()



from __future__ import annotations

from abc import abstractmethod
from collections.abc import Iterable
from typing import TYPE_CHECKING, Any

from checkov.arm.registry import arm_parameter_registry
from checkov.common.checks.base_check import BaseCheck

if TYPE_CHECKING:
    from checkov.common.models.enums import CheckCategories, CheckResult


class BaseParameterCheck(BaseCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: Iterable[CheckCategories],
        supported_resources: Iterable[str],
        guideline: str | None = None,
    ) -> None:
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_resources,
            block_type="parameter",
            guideline=guideline,
        )
        self.supported_resources = supported_resources
        arm_parameter_registry.register(self)

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> CheckResult:
        self.entity_type = entity_type

        return self.scan_resource_conf(conf)

    @abstractmethod
    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        raise NotImplementedError()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class CloudfrontDistributionEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure CloudFront distribution is enabled"
        id = "CKV_AWS_216"
        supported_resources = ['aws_cloudfront_distribution']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "enabled"


check = CloudfrontDistributionEnabled()



class PlatformConnectionError(Exception):
    def __init__(self, message: str) -> None:
        self.message = message

    def __str__(self) -> str:
        return f"PlatformConnectionError: {self.message}"


class BridgecrewAuthError(PlatformConnectionError):
    def __init__(self, message: str = "Authorization error accessing the platform api. Please check your API keys and "
                                      "Prisma API URL.") -> None:
        self.message = message

    def __str__(self) -> str:
        return f"BCAuthError: {self.message}"


class ModuleNotEnabledError(Exception):
    def __init__(self, message: str) -> None:
        self.message = message

    def __str__(self) -> str:
        return f"ModuleNotEnabledError: {self.message}"



from __future__ import annotations

import logging
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from argparse import Namespace
    from checkov.common.bridgecrew.integration_features.base_integration_feature import BaseIntegrationFeature
    from checkov.common.output.report import Report
    from checkov.common.typing import _BaseRunner


class IntegrationFeatureRegistry:
    def __init__(self) -> None:
        self.config: Namespace | None = None
        self.features: list[BaseIntegrationFeature] = []

    def register(self, integration_feature: BaseIntegrationFeature) -> None:
        logging.debug(f"Adding the IntegrationFeatureRegistry {integration_feature} with order {integration_feature.order}")
        self.features.append(integration_feature)
        self.features.sort(key=lambda f: f.order)
        logging.debug("self.features after the sort:")
        logging.debug(self.features)

    def run_pre_scan(self) -> None:
        for integration in self.features:
            if integration.is_valid():
                integration.config = self.config
                integration.pre_scan()

    def run_pre_runner(self, runner: _BaseRunner) -> None:
        for integration in self.features:
            if integration.is_valid():
                integration.pre_runner(runner)

    def run_post_runner(self, scan_report: Report) -> None:
        for integration in self.features:
            if integration.is_valid():
                integration.post_runner(scan_report)

    def run_post_scan(self, scan_reports: list[Report]) -> list[Report]:
        post_scan_reports = []
        for integration in self.features:
            if integration.is_valid():
                integration_report = integration.post_scan(scan_reports)
                if integration_report:
                    post_scan_reports.append(integration_report)

        return post_scan_reports


integration_feature_registry = IntegrationFeatureRegistry()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class ACRDedicatedDataEndpointEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure dedicated data endpoints are enabled."
        id = "CKV_AZURE_237"
        supported_resources = ("azurerm_container_registry",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "data_endpoint_enabled"


check = ACRDedicatedDataEndpointEnabled()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class SageMakerInternetAccessDisabled(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that direct internet access is disabled for an Amazon SageMaker Notebook Instance"
        id = "CKV_AWS_122"
        supported_resources = ['aws_sagemaker_notebook_instance']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self):
        return 'direct_internet_access'

    def get_expected_value(self):
        return 'Disabled'


check = SageMakerInternetAccessDisabled()



from __future__ import annotations

import logging
import os
from typing import Type, Any, TYPE_CHECKING, overload

from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR
from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph

from checkov.common.graph.graph_manager import GraphManager
from checkov.terraform.tf_parser import TFParser

if TYPE_CHECKING:
    from checkov.terraform.modules.module_objects import TFDefinitionKey
    from checkov.common.typing import LibraryGraphConnector


class TerraformGraphManager(GraphManager[TerraformLocalGraph, "dict[TFDefinitionKey, dict[str, Any]]"]):
    def __init__(self, db_connector: LibraryGraphConnector, source: str = "") -> None:
        self.parser: TFParser  # just to make sure it won't be None

        parser = TFParser()
        super().__init__(db_connector=db_connector, parser=parser, source=source)

    def build_multi_graph_from_source_directory(
        self,
        source_dir: str,
        local_graph_class: Type[TerraformLocalGraph] = TerraformLocalGraph,
        render_variables: bool = True,
        parsing_errors: dict[str, Exception] | None = None,
        download_external_modules: bool = False,
        excluded_paths: list[str] | None = None,
        external_modules_download_path: str = DEFAULT_EXTERNAL_MODULES_DIR,
        vars_files: list[str] | None = None,
    ) -> tuple[list[tuple[TerraformLocalGraph, list[dict[TFDefinitionKey, dict[str, Any]]], str]], dict[str, str]]:
        logging.info("Parsing HCL files in source dir to multi graph")
        modules_with_definitions = self.parser.parse_multi_graph_hcl_module(
            source_dir=source_dir,
            source=self.source,
            download_external_modules=download_external_modules,
            external_modules_download_path=external_modules_download_path,
            parsing_errors=parsing_errors,
            excluded_paths=excluded_paths,
            vars_files=vars_files,
        )

        graphs: list[tuple[TerraformLocalGraph, list[dict[TFDefinitionKey, dict[str, Any]]], str]] = []
        resource_subgraph_map: dict[str, str] = {}
        for module, tf_definitions in modules_with_definitions:
            logging.info("Building graph from parsed module")
            local_graph = local_graph_class(module)
            local_graph.build_graph(render_variables=render_variables)
            subgraph_abs_path = module.source_dir
            subgraph_path = subgraph_abs_path[subgraph_abs_path.rindex(source_dir) + len(source_dir) + 1:]
            graphs.append((local_graph, tf_definitions, subgraph_path))
            self.update_resource_subgraph_map(local_graph, subgraph_path, resource_subgraph_map, source_dir)
        return graphs, resource_subgraph_map

    def build_graph_from_source_directory(
        self,
        source_dir: str,
        local_graph_class: Type[TerraformLocalGraph] = TerraformLocalGraph,
        render_variables: bool = True,
        parsing_errors: dict[str, Exception] | None = None,
        download_external_modules: bool = False,
        excluded_paths: list[str] | None = None,
        external_modules_download_path: str = DEFAULT_EXTERNAL_MODULES_DIR,
        vars_files: list[str] | None = None,
    ) -> tuple[TerraformLocalGraph, dict[TFDefinitionKey, dict[str, Any]]]:
        logging.info("Parsing HCL files in source dir to graph")
        module, tf_definitions = self.parser.parse_hcl_module(
            source_dir=source_dir,
            source=self.source,
            download_external_modules=download_external_modules,
            external_modules_download_path=external_modules_download_path,
            parsing_errors=parsing_errors,
            excluded_paths=excluded_paths,
            vars_files=vars_files,
        )

        logging.info("Building graph from parsed module")
        local_graph = local_graph_class(module)
        local_graph.build_graph(render_variables=render_variables)

        return local_graph, tf_definitions

    @overload
    def build_graph_from_definitions(
        self, definitions: dict[str, dict[str, Any]], render_variables: bool = True,
    ) -> TerraformLocalGraph:
        ...

    @overload
    def build_graph_from_definitions(
        self, definitions: dict[TFDefinitionKey, dict[str, Any]], render_variables: bool = True,
    ) -> TerraformLocalGraph:
        ...

    def build_graph_from_definitions(
        self,
        definitions: dict[str, dict[str, Any]] | dict[TFDefinitionKey, dict[str, Any]],
        render_variables: bool = True,
    ) -> TerraformLocalGraph:
        module, _ = self.parser.parse_hcl_module_from_tf_definitions(definitions, "", self.source)
        local_graph = TerraformLocalGraph(module)
        local_graph.build_graph(render_variables=render_variables)

        return local_graph

    def build_multi_graph_from_definitions(
        self,
        definitions: dict[TFDefinitionKey, dict[str, Any]],
        render_variables: bool = True,
    ) -> list[tuple[str | None, TerraformLocalGraph]]:
        module, tf_definitions = self.parser.parse_hcl_module_from_tf_definitions(definitions, "", self.source)
        dirs_to_definitions = self.parser.create_definition_by_dirs(tf_definitions)

        graphs: list[tuple[str | None, TerraformLocalGraph]] = []
        for source_path, dir_definitions in dirs_to_definitions.items():
            module, parsed_tf_definitions = self.parser.parse_hcl_module_from_multi_tf_definitions(dir_definitions, source_path, self.source)
            local_graph = TerraformLocalGraph(module)
            local_graph.build_graph(render_variables=render_variables)
            graphs.append((source_path, local_graph))

        return graphs

    @staticmethod
    def update_resource_subgraph_map(
            local_graph: TerraformLocalGraph, subgraph_path: str, resource_subgraph_map: dict[str, str], source_dir: str
    ) -> None:
        for v in local_graph.vertices:
            resource_id = f"/{os.path.relpath(v.path, source_dir)}:{v.id}"
            resource_subgraph_map[resource_id] = subgraph_path



import os
from contextlib import contextmanager
from typing import Any, Generator


@contextmanager
def temp_environ(**kwargs: Any) -> Generator[None, None, None]:
    """Temporarily set environment variables and restores previous values

    copy of https://gist.github.com/igniteflow/7267431?permalink_comment_id=2553451#gistcomment-2553451
    """
    original_env = {key: os.getenv(key) for key in kwargs}
    os.environ.update(kwargs)
    try:
        yield
    finally:
        for key, value in original_env.items():
            if value is None:
                del os.environ[key]
            else:
                os.environ[key] = value



from checkov.common.models.enums import CheckCategories
from typing import List, Any
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class ACRAdminAccountDisabled(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure ACR admin account is disabled"
        id = "CKV_AZURE_137"
        supported_resources = ['azurerm_container_registry']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "admin_enabled"

    def get_forbidden_values(self) -> List[Any]:
        return [True]


check = ACRAdminAccountDisabled()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class DiskEncryptedWithCMK(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Disk is encrypted with Customer Master Key"
        id = "CKV_ALI_8"
        supported_resources = ("alicloud_disk",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if conf.get("snapshot_id"):
            return CheckResult.UNKNOWN
        encrypted = conf.get("encrypted")
        if encrypted and encrypted == [True]:
            if conf.get("kms_key_id"):
                return CheckResult.PASSED
        return CheckResult.FAILED


check = DiskEncryptedWithCMK()



from checkov.arm.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class AutomationEncrypted(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Automation account variables are encrypted"
        id = "CKV_AZURE_73"
        supported_resources = ("Microsoft.Automation/automationAccounts/variables",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/isEncrypted"

    def get_expected_value(self) -> bool:
        return True


check = AutomationEncrypted()



from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class DataExplorerUsesDiskEncryption(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure Data Explorer (Kusto) uses disk encryption"
        id = "CKV_AZURE_74"
        supported_resources = ("Microsoft.Kusto/clusters",)
        categories = [CheckCategories.ENCRYPTION,]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,)

    def get_inspected_key(self) -> str:
        return "properties/enableDiskEncryption"

    def get_expected_value(self) -> bool:
        return True


check = DataExplorerUsesDiskEncryption()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class GoogleKMSPreventDestroy(BaseResourceValueCheck):
    def __init__(self):
        # "From the Provider Documentation"
        # CryptoKeys cannot be deleted from Google Cloud Platform.Destroying a Terraform - managed CryptoKey will remove
        # it from state and delete all CryptoKeyVersions, rendering the key unusable, but will not delete the resource
        # from the project.When Terraform destroys these keys, any data previously encrypted with these keys will be
        # irrecoverable.For this reason, it is strongly recommended that you add lifecycle hooks to the resource to
        # prevent accidental destruction.
        name = "Ensure KMS keys are protected from deletion"
        id = "CKV_GCP_82"
        supported_resources = ['google_kms_crypto_key']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'lifecycle/[0]/prevent_destroy'


check = GoogleKMSPreventDestroy()



#!/usr/bin/env python

from __future__ import annotations

import os
import re
import inspect
from typing import List, Optional, Tuple, Union

from tabulate import tabulate

from checkov.ansible.checks.registry import registry as ansible_registry
from checkov.argo_workflows.checks.registry import registry as argo_workflows_registry
from checkov.arm.registry import arm_resource_registry, arm_parameter_registry
from checkov.azure_pipelines.checks.registry import registry as azure_pipelines_registry
from checkov.bicep.checks.param.registry import registry as bicep_param_registry
from checkov.bicep.checks.resource.registry import registry as bicep_resource_registry
from checkov.bitbucket.registry import registry as bitbucket_configuration_registry
from checkov.bitbucket_pipelines.registry import registry as bitbucket_pipelines_registry
from checkov.circleci_pipelines.registry import registry as circleci_pipelines_registry
from checkov.cloudformation.checks.resource.registry import cfn_registry as cfn_registry
from checkov.common.checks.base_check_registry import BaseCheckRegistry
from checkov.common.checks_infra.registry import BaseRegistry as BaseGraphRegistry, get_graph_checks_registry
from checkov.common.runners.base_runner import strtobool
from checkov.dockerfile.registry import registry as dockerfile_registry
from checkov.github.registry import registry as github_configuration_registry
from checkov.github_actions.checks.registry import registry as github_actions_jobs_registry
from checkov.gitlab.registry import registry as gitlab_configuration_registry
from checkov.gitlab_ci.checks.registry import registry as gitlab_ci_jobs_registry
from checkov.kubernetes.checks.resource.registry import registry as k8_registry
from checkov.secrets.runner import CHECK_ID_TO_SECRET_TYPE
from checkov.serverless.registry import sls_registry
from checkov.terraform.checks.data.registry import data_registry
from checkov.terraform.checks.module.registry import module_registry
from checkov.terraform.checks.provider.registry import provider_registry
from checkov.terraform.checks.resource.registry import resource_registry
from checkov.openapi.checks.registry import openapi_registry
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import integration as metadata_integration
from checkov.runner_filter import RunnerFilter

ID_PARTS_PATTERN = re.compile(r'([^_]*)_([^_]*)_(\d+)')
CODE_LINK_BASE = 'https://github.com/bridgecrewio/checkov/blob/main/checkov'
CREATE_MARKDOWN_HYPERLINKS = strtobool(os.getenv("CHECKOV_CREATE_MARKDOWN_HYPERLINKS", "FALSE"))
SKIP_CHECK_IDS = {
    "CKV_SECRET_10",  # this is an intermediate step, which is needed for another check
}


def get_compare_key(c: list[str] | tuple[str, ...]) -> list[tuple[str, str, int, int, str]]:
    res = []
    for match in ID_PARTS_PATTERN.finditer(c[0]):
        ckv, framework, number = match.groups()
        numeric_value = int(number) if number else 0
        # count number of leading zeros
        same_number_ordering = len(number) - len(number.lstrip('0'))
        res.append((framework, ckv, numeric_value, same_number_ordering, c[2]))
    return res


def print_checks(frameworks: Optional[List[str]] = None, use_bc_ids: bool = False,
                 include_all_checkov_policies: bool = True, filtered_policy_ids: Optional[List[str]] = None,
                 filtered_exception_policy_ids: Optional[List[str]] = None) -> None:
    framework_list = frameworks if frameworks else ["all"]
    printable_checks_list = get_checks(framework_list, use_bc_ids=use_bc_ids,
                                       include_all_checkov_policies=include_all_checkov_policies,
                                       filtered_policy_ids=filtered_policy_ids or [],
                                       filtered_exception_policy_ids=filtered_exception_policy_ids or [])
    print(
        tabulate(printable_checks_list, headers=["Id", "Type", "Entity", "Policy", "IaC", "Resource Link"], tablefmt="github",
                 showindex=True))
    print("\n\n---\n\n")


def get_check_link(absolute_path: str) -> str:
    # this will do nothing unless it's a windows path
    absolute_path = absolute_path.replace('\\', '/')
    temp = absolute_path.split("checkov")
    # this will even work in the likely event that you're running checkov from a folder called checkov
    link = f'{CODE_LINK_BASE}{temp[len(temp)-1]}'

    if CREATE_MARKDOWN_HYPERLINKS:
        return f"[{absolute_path.rsplit('/', maxsplit=1)[1]}]({link})"

    return link


def get_checks(frameworks: Optional[List[str]] = None, use_bc_ids: bool = False,
               include_all_checkov_policies: bool = True, filtered_policy_ids: Optional[List[str]] = None,
               filtered_exception_policy_ids: Optional[List[str]] = None) -> List[Tuple[str, str, int, int, str, str]]:
    framework_list = frameworks if frameworks else ["all"]
    printable_checks_list: list[tuple[str, str, str, str, str, str]] = []
    filtered_policy_ids = filtered_policy_ids or []
    filtered_exception_policy_ids = filtered_exception_policy_ids or []
    runner_filter = RunnerFilter(include_all_checkov_policies=include_all_checkov_policies,
                                 filtered_policy_ids=filtered_policy_ids,
                                 filtered_exception_policy_ids=filtered_exception_policy_ids)

    def add_from_repository(registry: Union[BaseCheckRegistry, BaseGraphRegistry], checked_type: str, iac: str,
                            runner_filter: RunnerFilter = runner_filter) -> None:
        nonlocal printable_checks_list
        if isinstance(registry, BaseCheckRegistry):
            for entity, check in registry.all_checks():
                if runner_filter.should_run_check(check, check.id, check.bc_id, check.severity):
                    check_link = get_check_link(inspect.getfile(check.__class__))
                    printable_checks_list.append(
                        (check.get_output_id(use_bc_ids), checked_type, entity, check.name, iac, check_link))
        elif isinstance(registry, BaseGraphRegistry):
            for graph_check in registry.checks:
                if runner_filter.should_run_check(graph_check, graph_check.id, graph_check.bc_id, graph_check.severity):
                    if not graph_check.resource_types:
                        # only for platform custom polices with resource_types == all
                        graph_check.resource_types = ['all']
                    for rt in graph_check.resource_types:
                        if graph_check.check_path:
                            base_path = graph_check.check_path
                        else:
                            base_path = inspect.getfile(graph_check.__class__)
                        check_link = get_check_link(base_path)
                        printable_checks_list.append(
                            (graph_check.get_output_id(use_bc_ids), checked_type, rt, graph_check.name, iac, check_link))

    if any(x in framework_list for x in ("all", "terraform")):
        add_from_repository(resource_registry, "resource", "Terraform")
        add_from_repository(data_registry, "data", "Terraform")
        add_from_repository(provider_registry, "provider", "Terraform")
        add_from_repository(module_registry, "module", "Terraform")

        graph_registry = get_graph_checks_registry("terraform")
        graph_registry.load_checks()
        add_from_repository(graph_registry, "resource", "Terraform")
    if any(x in framework_list for x in ("all", "cloudformation")):
        graph_registry = get_graph_checks_registry("cloudformation")
        graph_registry.load_checks()
        add_from_repository(graph_registry, "resource", "Cloudformation")
        add_from_repository(cfn_registry, "resource", "Cloudformation")
    if any(x in framework_list for x in ("all", "kubernetes")):
        graph_registry = get_graph_checks_registry("kubernetes")
        graph_registry.load_checks()
        add_from_repository(graph_registry, "resource", "Kubernetes")
        add_from_repository(k8_registry, "resource", "Kubernetes")
    if any(x in framework_list for x in ("all", "serverless")):
        add_from_repository(sls_registry, "resource", "serverless")
    if any(x in framework_list for x in ("all", "dockerfile")):
        graph_registry = get_graph_checks_registry("dockerfile")
        graph_registry.load_checks()
        add_from_repository(graph_registry, "resource", "dockerfile")
        add_from_repository(dockerfile_registry, "dockerfile", "dockerfile")
    if any(x in framework_list for x in ("all", "github_configuration")):
        add_from_repository(github_configuration_registry, "github_configuration", "github_configuration")
    if any(x in framework_list for x in ("all", "github_actions")):
        graph_registry = get_graph_checks_registry("github_actions")
        graph_registry.load_checks()
        add_from_repository(graph_registry, "resource", "github_actions")
        add_from_repository(github_actions_jobs_registry, "jobs", "github_actions")
    if any(x in framework_list for x in ("all", "gitlab_ci")):
        add_from_repository(gitlab_ci_jobs_registry, "jobs", "gitlab_ci")
    if any(x in framework_list for x in ("all", "gitlab_configuration")):
        add_from_repository(gitlab_configuration_registry, "gitlab_configuration", "gitlab_configuration")
    if any(x in framework_list for x in ("all", "bitbucket_configuration")):
        add_from_repository(bitbucket_configuration_registry, "bitbucket_configuration", "bitbucket_configuration")
    if any(x in framework_list for x in ("all", "bitbucket_pipelines")):
        add_from_repository(bitbucket_pipelines_registry, "bitbucket_pipelines", "bitbucket_pipelines")
    if any(x in framework_list for x in ("all", "circleci_pipelines")):
        add_from_repository(circleci_pipelines_registry, "circleci_pipelines", "circleci_pipelines")
    if any(x in framework_list for x in ("all", "argo_workflows")):
        add_from_repository(argo_workflows_registry, "argo_workflows", "Argo Workflows")
    if any(x in framework_list for x in ("all", "azure_pipelines")):
        add_from_repository(azure_pipelines_registry, "azure_pipelines", "Azure Pipelines")
    if any(x in framework_list for x in ("all", "arm")):
        graph_registry = get_graph_checks_registry("arm")
        graph_registry.load_checks()
        add_from_repository(graph_registry, "resource", "arm")
        add_from_repository(arm_resource_registry, "resource", "arm")
        add_from_repository(arm_parameter_registry, "parameter", "arm")
    if any(x in framework_list for x in ("all", "bicep")):
        graph_registry = get_graph_checks_registry("bicep")
        graph_registry.load_checks()
        add_from_repository(graph_registry, "resource", "Bicep")
        add_from_repository(bicep_param_registry, "parameter", "Bicep")
        add_from_repository(bicep_resource_registry, "resource", "Bicep")
    if any(x in framework_list for x in ("all", "openapi")):
        add_from_repository(openapi_registry, "resource", "OpenAPI")
    if any(x in framework_list for x in ("all", "ansible")):
        graph_registry = get_graph_checks_registry("ansible")
        graph_registry.load_checks()
        add_from_repository(graph_registry, "resource", "Ansible")
        add_from_repository(ansible_registry, "resource", "Ansible")
    if any(x in framework_list for x in ("all", "secrets")):
        for check_id, check_type in CHECK_ID_TO_SECRET_TYPE.items():
            if check_id in SKIP_CHECK_IDS:
                continue

            if not filtered_policy_ids or check_id in filtered_policy_ids:
                if use_bc_ids:
                    check_id = metadata_integration.get_bc_id(check_id)
                check_link = get_check_link(inspect.getfile(metadata_integration.__class__))
                printable_checks_list.append((check_id, check_type, "secrets", check_type, "secrets", check_link))
    return sorted(printable_checks_list, key=get_compare_key)  # type:ignore[arg-type]


if __name__ == '__main__':
    print_checks()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


class SecurityCenterContactEmailAlert(BaseResourceCheck):
    def __init__(self) -> None:
        # https://docs.microsoft.com/en-us/azure/templates/microsoft.security/securitycontacts
        name = "Ensure that 'Send email notification for high severity alerts' is set to 'On'"
        id = "CKV_AZURE_21"
        supported_resources = ('Microsoft.Security/securityContacts',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        if "properties" in conf:
            if "alertNotifications" in conf["properties"]:
                if str(conf["properties"]["alertNotifications"]).lower() == "on":
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = SecurityCenterContactEmailAlert()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_parameter_check import BaseParameterCheck


class SecureStringParameterNoHardcodedValue(BaseParameterCheck):
    def __init__(self) -> None:
        name = "SecureString parameter should not have hardcoded default values"
        id = "CKV_AZURE_131"
        supported_resources = ('secureString',)
        categories = (CheckCategories.SECRETS,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        # https://docs.microsoft.com/en-us/azure/azure-resource-manager/templates/test-cases#secure-parameters-cant-have-hardcoded-default
        default_value = conf.get('defaultValue')
        if default_value:  # should be missing, or an empty string
            conf[f'{self.id}_secret'] = default_value
            return CheckResult.FAILED
        else:
            return CheckResult.PASSED


check = SecureStringParameterNoHardcodedValue()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.gitlab.base_gitlab_configuration_check import BaseGitlabCheck
from checkov.gitlab.schemas.groups import schema
from checkov.json_doc.enums import BlockType


class GroupsTwoFactorAuthentication(BaseGitlabCheck):
    def __init__(self) -> None:
        name = "Ensure all Gitlab groups require two factor authentication"
        id = "CKV_GITLAB_2"
        categories = [CheckCategories.SUPPLY_CHAIN]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=["*"],
            block_type=BlockType.DOCUMENT
        )

    def scan_entity_conf(self, conf: list[dict[str, Any]], entity_type: str) -> CheckResult | None:  # type:ignore[override]
        if schema.validate(conf):
            for group in conf:
                if group.get("require_two_factor_authentication") is False:
                    return CheckResult.FAILED
            return CheckResult.PASSED
        return None


check = GroupsTwoFactorAuthentication()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class RootContainersPSP(BaseResourceCheck):

    def __init__(self):
        # CIS-1.3 1.7.6
        # CIS-1.5 5.2.6
        name = "Do not admit root containers"
        id = "CKV_K8S_6"
        supported_resources = ['kubernetes_pod_security_policy']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        # required param
        if "spec" in conf:
            # required param
            if "run_as_user" in conf["spec"][0]:
                runas = conf["spec"][0]["run_as_user"][0]
                if runas["rule"]:
                    inspected_value = runas["rule"][0]
                    if inspected_value == "MustRunAsNonRoot":
                        return CheckResult.PASSED
                    elif inspected_value == "MustRunAs":
                        if runas["range"]:
                            for item in runas["range"]:
                                if item["min"][0] == 0:
                                    return CheckResult.FAILED
                            return CheckResult.PASSED
        return CheckResult.FAILED


check = RootContainersPSP()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class FunctionAppsAccessibleOverHttps(BaseResourceCheck):

    def __init__(self) -> None:
        name = "Ensure that Function apps is only accessible over HTTPS"
        id = "CKV_AZURE_70"
        supported_resources = ['azurerm_function_app', 'azurerm_linux_function_app', 'azurerm_windows_function_app',
                               'azurerm_function_app_slot', 'azurerm_linux_function_app_slot',
                               'azurerm_windows_function_app_slot']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        # default=false for https_only
        if 'https_only' not in conf.keys():
            return CheckResult.FAILED

        https_only = conf.get('https_only')[0]
        if not https_only:
            return CheckResult.FAILED

        # relevant for linux/windows resources
        if 'auth_settings_v2' in conf.keys():
            auth_settings_v2 = conf['auth_settings_v2'][0]

            # default=true for require_https
            if 'require_https' not in auth_settings_v2.keys():
                return CheckResult.PASSED

            require_https = auth_settings_v2.get('require_https')[0]
            if not require_https:
                return CheckResult.FAILED

        return CheckResult.PASSED


check = FunctionAppsAccessibleOverHttps()



from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class SagemakerModelWithNetworkIsolation(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Amazon SageMaker model uses network isolation"
        id = "CKV_AWS_370"
        supported_resources = ["AWS::SageMaker::Model"]
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'Properties/EnableNetworkIsolation'


check = SagemakerModelWithNetworkIsolation()



from __future__ import annotations

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AppServicePlanZoneRedundant(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        To enhance the resiliency and reliability of business-critical workloads,
        it's recommended to deploy new App Service Plans with zone-redundancy.

        There's no additional cost associated with enabling availability zones.
        Pricing for a zone redundant App Service is the same as a single zone App Service.
        """
        name = "Ensure the App Service Plan is zone redundant"
        id = "CKV_AZURE_225"
        supported_resources = ("azurerm_service_plan",)
        categories = (CheckCategories.BACKUP_AND_RECOVERY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "zone_balancing_enabled"


check = AppServicePlanZoneRedundant()



from __future__ import annotations

import re
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from detect_secrets.util.code_snippet import CodeSnippet

from checkov.secrets.parsers.multiline_parser import BaseMultiLineParser

INDENTATION_PATTERN = re.compile(r'(^\s*(?:-?\s+)?)')
COMMENT_PREFIX = re.compile(r'^[\s]*(#|\/\/)')


class YmlMultilineParser(BaseMultiLineParser):

    def consecutive_lines_in_same_object(
        self,
        raw_context: CodeSnippet | None,
        other_line_idx: int,
    ) -> bool:
        if not raw_context:
            return False  # could not know
        return 0 <= other_line_idx < len(raw_context.lines) and \
            self.lines_same_indentation(raw_context.lines[other_line_idx], raw_context.target_line)

    @staticmethod
    def is_object_start(
        line: str,
    ) -> bool:
        match = re.match(INDENTATION_PATTERN, line)
        if match:
            return '-' in match.groups()[0]
        return False

    @staticmethod
    def is_object_end(
        line: str,
    ) -> bool:
        match = re.match(INDENTATION_PATTERN, line)
        if match:
            return '-' in match.groups()[0]
        return False

    @staticmethod
    def is_line_comment(
        line: str
    ) -> bool:
        return bool(re.match(COMMENT_PREFIX, line))

    @staticmethod
    def lines_same_indentation(line1: str, line2: str) -> bool:
        match1 = re.match(INDENTATION_PATTERN, line1)
        match2 = re.match(INDENTATION_PATTERN, line2)
        if not match1 and not match2:
            return True
        if not match1 or not match2:
            return False
        indent1 = len(match1.groups()[0])
        indent2 = len(match2.groups()[0])
        return indent1 == indent2


yml_multiline_parser = YmlMultilineParser()



from __future__ import annotations

import json
import logging
from typing import TYPE_CHECKING, Any

from checkov.common.bridgecrew.code_categories import CodeCategoryConfiguration, CodeCategoryType
from checkov.common.bridgecrew.integration_features.base_integration_feature import BaseIntegrationFeature
from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.bridgecrew.severities import Severities, BcSeverities

if TYPE_CHECKING:
    from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
    from checkov.common.output.report import Report
    from checkov.common.typing import _BaseRunner


class RepoConfigIntegration(BaseIntegrationFeature):
    def __init__(self, bc_integration: BcPlatformIntegration) -> None:
        super().__init__(bc_integration=bc_integration, order=0)
        self.skip_paths: set[str] = set()
        self.enforcement_rule: dict[str, Any] = {}
        self.code_category_configs: dict[str, CodeCategoryConfiguration] = {}

    def is_valid(self) -> bool:
        return (
            self.bc_integration.is_integration_configured()
            and not self.bc_integration.skip_download
            and not self.integration_feature_failures
        )

    def pre_scan(self) -> None:
        try:
            if not self.bc_integration.customer_run_config_response:
                logging.debug('In the pre-scan for repo config settings, but nothing was fetched from the platform')
                self.integration_feature_failures = True
                return

            # It is possible that they will have two different and conflicting rules for this repo - one for the VCS
            # integration that matches the value of --repo-id (org/repo), and one for the CLI upload repo (e.g., customer_org/repo).
            # For the skip paths, we can just combine the lists and call it good. For enforcement rules, we will
            # prioritize the VCS integration over CLI, and warn them that the rules should match for these repos.

            self._set_exclusion_paths(self.bc_integration.customer_run_config_response['vcsConfig'])
            self._set_enforcement_rules(self.bc_integration.customer_run_config_response['enforcementRules'])

        except Exception:
            self.integration_feature_failures = True
            logging.debug("Scanning without applying scanning configs from the platform.", exc_info=True)

    @staticmethod
    def _get_code_category_object(code_category_config: dict[str, Any],
                                  code_category_type: str) -> CodeCategoryConfiguration | None:
        if code_category_type not in code_category_config:
            return None
        soft_fail_threshold = Severities[code_category_config[code_category_type]['softFailThreshold']]
        hard_fail_threshold = Severities[code_category_config[code_category_type]['hardFailThreshold']]
        return CodeCategoryConfiguration(code_category_type, soft_fail_threshold, hard_fail_threshold)

    def _set_exclusion_paths(self, vcs_config: dict[str, Any]) -> None:
        for section in vcs_config['scannedFiles']['sections']:
            repos = section['repos']
            if any(repo for repo in repos if self.bc_integration.repo_matches(repo)):
                logging.debug(f'Found path exclusion config section for repo: {section}')
                self.skip_paths.update(section['rule']['excludePaths'])

        logging.debug(f'Skipping the following paths based on platform settings: {self.skip_paths}')

    def _set_enforcement_rules(self, enforcement_rules_config: dict[str, Any]) -> None:
        rules = enforcement_rules_config['rules']
        default_rule = next(r for r in rules if r['mainRule'] is True)
        other_rules = [r for r in rules if r != default_rule]

        logging.debug(f'Default enforcement rule: {json.dumps(default_rule, indent=2)}')
        logging.debug(f'Other enforcement rules ({len(other_rules)} total): {json.dumps(other_rules, indent=2)}')

        matched_rules = []

        for rule in other_rules:
            if any(repo for repo in rule['repositories'] if self.bc_integration.repo_matches(repo['accountName'])):
                matched_rules.append(rule)

        if len(matched_rules) > 1:
            logging.warning(f'Found {len(matched_rules)} enforcement rules for the specified repo. This likely means '
                            f'that one rule was created for the VCS repo, and another rule for the CLI repo. You '
                            f'should update the configurations in the platform to ensure that the following repos '
                            f'are all in the same rule group:')
            exact_match_rule = None
            for rule in matched_rules:
                for repo in rule['repositories']:
                    repo_name = repo['accountName']
                    if self.bc_integration.repo_matches(repo_name):
                        logging.warning(f'- {repo_name}')
                        if repo_name == self.bc_integration.repo_id:
                            if exact_match_rule:
                                logging.debug('Found multiple rules that exactly match --repo-id - likely the same '
                                              'name across multiple VCSes. Using the first one.')
                            else:
                                exact_match_rule = rule

            if not exact_match_rule:
                logging.debug('Did not find any rules with a repo name that exactly matched --repo-id; taking the '
                              'first one.')

            self.enforcement_rule = exact_match_rule or matched_rules[0]
        elif len(matched_rules) == 0:
            logging.info('Did not find any enforcement rules for the specified repo; using the default rule')
            self.enforcement_rule = default_rule
        else:
            logging.info('Found exactly one matching enforcement rule for the specified repo')
            self.enforcement_rule = matched_rules[0]

        logging.debug(
            'Selected the following enforcement rule (it will not be applied unless --use-enforcement-rules is specified):')
        logging.debug(json.dumps(self.enforcement_rule, indent=2))

        # handle the rename in the platform side
        if 'SAST' in self.enforcement_rule['codeCategories'] and 'WEAKNESSES' not in self.enforcement_rule['codeCategories']:
            self.enforcement_rule['codeCategories']['WEAKNESSES'] = self.enforcement_rule['codeCategories']['SAST']

        if 'WEAKNESSES' in self.enforcement_rule['codeCategories'] and 'SAST' not in self.enforcement_rule['codeCategories']:
            self.enforcement_rule['codeCategories']['SAST'] = self.enforcement_rule['codeCategories']['WEAKNESSES']

        for code_category_type in [e.value for e in CodeCategoryType]:
            config = RepoConfigIntegration._get_code_category_object(self.enforcement_rule['codeCategories'],
                                                                     code_category_type)
            if config:
                self.code_category_configs[code_category_type] = config

    def is_code_review_disabled(self, code_category_type: str) -> bool:
        config = self.code_category_configs[code_category_type]
        if config.hard_fail_threshold == Severities[BcSeverities.OFF] and config.soft_fail_threshold == Severities[BcSeverities.OFF]:
            return True
        return False

    def pre_runner(self, runner: _BaseRunner) -> None:
        # not used
        pass

    def post_runner(self, scan_reports: Report) -> None:
        # not used
        pass

    def post_scan(self, merged_reports: list[Report]) -> None:
        # not used
        pass


integration = RepoConfigIntegration(bc_integration)



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class NeptuneDBClustersIAMDatabaseAuthenticationEnabled(BaseResourceValueCheck):
    def __init__(self):
        description = "Neptune DB clusters should have IAM database authentication enabled"
        id = "CKV_AWS_359"
        supported_resources = ['aws_neptune_cluster']
        categories = [CheckCategories.IAM]
        super().__init__(name=description, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "iam_database_authentication_enabled"


check = NeptuneDBClustersIAMDatabaseAuthenticationEnabled()



from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class CognitoUnauthenticatedIdentities(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure AWS Cognito identity pool does not allow unauthenticated guest access"
        id = "CKV_AWS_366"
        supported_resources = ('AWS::Cognito::IdentityPool',)
        categories = (CheckCategories.IAM,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_block_result=CheckResult.FAILED,
        )

    def get_expected_value(self) -> Any:
        return False

    def get_inspected_key(self) -> str:
        return 'Properties/AllowUnauthenticatedIdentities'


check = CognitoUnauthenticatedIdentities()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class CloudtrailEventDataStoreUsesCMK(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure CloudTrail Event Data Store uses CMK"
        id = "CKV_AWS_294"
        supported_resources = ['aws_cloudtrail_event_data_store']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'kms_key_id'

    def get_expected_value(self):
        return ANY_VALUE


check = CloudtrailEventDataStoreUsesCMK()



# the order is important, because it reflects the order, which will be used to download the module
from checkov.terraform.module_loading.loaders.registry_loader import RegistryLoader  # noqa
from checkov.terraform.module_loading.loaders.git_loader import GenericGitLoader  # noqa
from checkov.terraform.module_loading.loaders.github_loader import GithubLoader  # noqa
from checkov.terraform.module_loading.loaders.bitbucket_loader import BitbucketLoader  # noqa
from checkov.terraform.module_loading.loaders.github_access_token_loader import GithubAccessTokenLoader  # noqa
from checkov.terraform.module_loading.loaders.bitbucket_access_token_loader import BitbucketAccessTokenLoader  # noqa
from checkov.terraform.module_loading.loaders.local_path_loader import LocalPathLoader  # noqa



from typing import Any, Dict

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class ControllerManagerBindAddress(BaseK8sContainerCheck):
    def __init__(self) -> None:
        id = "CKV_K8S_113"
        name = "Ensure that the --bind-address argument is set to 127.0.0.1"
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["command"]
        if conf.get("command"):
            if "kube-controller-manager" in conf["command"]:
                for cmd in conf["command"]:
                    if "=" in cmd:
                        [key, value, *_] = cmd.split("=")
                        if key == "--bind-address" and value == "127.0.0.1":
                            return CheckResult.PASSED
                return CheckResult.FAILED

        return CheckResult.PASSED


check = ControllerManagerBindAddress()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck


class DocDBTLS(BaseResourceCheck):
    def __init__(self):
        name = "Ensure DocumentDB TLS is not disabled"
        id = "CKV_AWS_90"
        supported_resources = ['aws_docdb_cluster_parameter_group']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        self.evaluated_keys = ['parameter']
        if 'parameter' in conf:
            for idx, elem in enumerate(conf["parameter"]):
                if isinstance(elem, dict) and elem["name"][0] == "tls" and elem["value"][0] == "disabled":
                    self.evaluated_keys = [f'parameter/[{idx}]/name', f'parameter/[{idx}]/value']
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = DocDBTLS()



from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class SagemakerNotebookInCustomVPC(BaseResourceValueCheck):
    def __init__(self):
        """
        NIST.800-53.r5 AC-21, NIST.800-53.r5 AC-3, NIST.800-53.r5 AC-3(7), NIST.800-53.r5 AC-4, NIST.800-53.r5 AC-4(21),
        NIST.800-53.r5 AC-6, NIST.800-53.r5 SC-7, NIST.800-53.r5 SC-7(11), NIST.800-53.r5 SC-7(16),
        NIST.800-53.r5 SC-7(20), NIST.800-53.r5 SC-7(21), NIST.800-53.r5 SC-7(3), NIST.800-53.r5 SC-7(4),
        NIST.800-53.r5 SC-7(9)
        """
        name = "Ensure SageMaker notebook instances should be launched into a custom VPC"
        id = "CKV_AWS_306"
        supported_resources = ['aws_sagemaker_notebook_instance']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'subnet_id'

    def get_expected_value(self):
        return ANY_VALUE


check = SagemakerNotebookInCustomVPC()



from checkov.terraform.checks.resource.oci.AbsSecurityListUnrestrictedIngress import AbsSecurityListUnrestrictedIngress


class SecurityListUnrestrictedIngress22(AbsSecurityListUnrestrictedIngress):
    def __init__(self):
        super().__init__(check_id="CKV_OCI_19", port=22, is_exposed_by_default=True)


check = SecurityListUnrestrictedIngress22()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class SesConfigurationSetDefinesTLS(BaseResourceValueCheck):

    def __init__(self) -> None:
        name = "Ensure SES Configuration Set enforces TLS usage"
        id = "CKV_AWS_365"
        supported_resources = ['aws_ses_configuration_set']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.FAILED)

    def get_inspected_key(self) -> str:
        return "delivery_options/[0]/tls_policy"

    def get_expected_value(self) -> str:
        return "Require"


check = SesConfigurationSetDefinesTLS()



from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class AppServiceDotnetFrameworkVersion(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that 'Net Framework' version is the latest, if used as a part of the web app"
        id = "CKV_AZURE_80"
        supported_resources = ['Microsoft.Web/sites/config']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/netFrameworkVersion"

    def get_expected_value(self) -> str:
        return "v8.0"


check = AppServiceDotnetFrameworkVersion()



from enum import Enum
from typing import List, Any, Set
from pathlib import Path


SAST_FRAMEWORK_PREFIX = 'sast'
CDK_FRAMEWORK_PREFIX = 'cdk'
# checkov/checkov/cdk/checks
CDK_CHECKS_DIR_PATH = Path(__file__).parent.parent.parent / CDK_FRAMEWORK_PREFIX / "checks"


class SastLanguages(Enum):
    @classmethod
    def list(cls) -> List[Any]:
        return list(map(lambda c: c.value, cls))

    @classmethod
    def set(cls) -> Set["SastLanguages"]:
        return set(cls)

    PYTHON = 'python'
    JAVA = 'java'
    JAVASCRIPT = 'javascript'
    TYPESCRIPT = 'typescript'
    GOLANG = 'golang'


class CDKLanguages(Enum):
    @classmethod
    def list(cls) -> List[Any]:
        return list(map(lambda c: c.value, cls))

    @classmethod
    def set(cls) -> Set["CDKLanguages"]:
        return set(cls)

    PYTHON = 'python'
    TYPESCRIPT = 'typescript'


class BqlVersion(str, Enum):
    def __str__(self) -> str:
        return self.value

    V0_1 = '0.1'
    V0_2 = '0.2'


def get_bql_version_from_string(version_str: str) -> str:
    for version in BqlVersion:
        if version.value == version_str:
            return version
    return ''


SUPPORT_FILE_EXT = {
    SastLanguages.PYTHON: ['py'],
    SastLanguages.JAVA: ['java'],
    SastLanguages.JAVASCRIPT: ['js'],
    SastLanguages.TYPESCRIPT: ['ts'],
    SastLanguages.GOLANG: ['go'],
}

FILE_EXT_TO_SAST_LANG = {
    'py': SastLanguages.PYTHON,
    'java': SastLanguages.JAVA,
    'js': SastLanguages.JAVASCRIPT,
    'ts': SastLanguages.TYPESCRIPT,
    'go': SastLanguages.GOLANG,
}

POLICIES_ERRORS = 'policies_errors'
POLICIES_ERRORS_COUNT = 'policies_errors_count'
ENGINE_NAME = "engine_name"
SOURCE_FILES_COUNT = "source_files_count"
POLICY_COUNT = "policy_count"



from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck
from typing import List
from typing import Any


class AppServiceFTPSState(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure FTP deployments are disabled"
        id = "CKV_AZURE_78"
        supported_resources = ('Microsoft.Web/sites',)
        categories = (CheckCategories.APPLICATION_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "siteConfig/ftpsState"

    def get_expected_value(self) -> Any:
        return "Disabled"

    def get_expected_values(self) -> List[Any]:
        return ["Disabled", "FtpsOnly"]


check = AppServiceFTPSState()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_value_check import BaseResourceValueCheck
from typing import Dict, Any


class KeyVaultDisablesPublicNetworkAccess(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure Key Vault disables public network access"
        id = "CKV_AZURE_189"
        supported_resources = ("Microsoft.KeyVault/vaults",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "publicNetworkAccess"

    def get_expected_value(self) -> str:
        return "disabled"

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        properties = conf.get("properties", {})
        if self.get_inspected_key() in properties:
            conf_value = conf["properties"][self.get_inspected_key()]
            if conf_value and conf_value == self.get_expected_value():
                return CheckResult.PASSED

        if properties and "networkAcls" in properties:
            network_acls = conf["properties"]["networkAcls"]
            if isinstance(network_acls, dict) and "ipRules" in network_acls:
                ip_rules = network_acls["ipRules"]
                ip_rules = ip_rules[0] if ip_rules and isinstance(ip_rules, list) else ip_rules
                if ip_rules:
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = KeyVaultDisablesPublicNetworkAccess()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.util.data_structures_utils import find_in_dict


class DatabricksWorkspaceIsNotPublic(BaseResourceCheck):
    def __init__(self) -> None:
        # https://learn.microsoft.com/en-us/azure/templates/microsoft.databricks/workspaces?pivots=deployment-language-arm-template
        name = "Ensure Databricks Workspace data plane to control plane communication happens over private link"
        id = "CKV_AZURE_158"
        supported_resources = ("Microsoft.Databricks/workspaces",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        public_network_access = find_in_dict(input_dict=conf, key_path="properties/publicNetworkAccess")
        if not public_network_access or public_network_access == "Enabled":
            return CheckResult.FAILED

        return CheckResult.PASSED


check = DatabricksWorkspaceIsNotPublic()



from __future__ import annotations

from typing import Any

from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class AppServiceUsedAzureFiles(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that app services use Azure Files"
        id = "CKV_AZURE_88"
        supported_resources = ("Microsoft.Web/sites/config",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        properties = conf.get('properties')
        if properties and isinstance(properties, dict):
            azureStorageAccounts = properties.get("azureStorageAccounts")
            if azureStorageAccounts and isinstance(azureStorageAccounts, dict):
                for account_data in azureStorageAccounts.values():
                    if isinstance(account_data, dict) and account_data.get('type') == "AzureFiles":
                        return CheckResult.PASSED
        return CheckResult.FAILED


check = AppServiceUsedAzureFiles()



from typing import Any

from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class GKEDontUseNodePools(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        """
        Node pools are better configured in the separate resource google_container_node_pool
        or to quote the provider docs:
        "It is recommended that node pools be created and managed as separate resources as in the example above.
        This allows node pools to be added and removed without recreating the cluster.
        Node pools defined directly in the google_container_cluster resource cannot be removed
        without re-creating the cluster."
        Recreating a cluster in Production would be unwise.
        """

        name = "GKE Don't Use NodePools in the Cluster configuration"
        id = "CKV_GCP_123"
        supported_resources = ['google_container_cluster',]
        categories = [CheckCategories.KUBERNETES,]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'node_pool'

    def get_forbidden_values(self) -> Any:
        return [ANY_VALUE]


check = GKEDontUseNodePools()



from checkov.arm.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class FunctionAppsEnableAuthentication(BaseResourceValueCheck):

    def __init__(self) -> None:
        name = "Ensure that function apps enables Authentication"
        id = "CKV_AZURE_56"
        supported_resources = ("Microsoft.Web/sites/config",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name,
                         id=id,
                         categories=categories,
                         supported_resources=supported_resources,

                         )

    def get_inspected_key(self) -> str:
        return 'properties/platform/enabled'


check = FunctionAppsEnableAuthentication()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class SeccompPSP(BaseResourceCheck):

    def __init__(self):
        # CIS-1.5 5.7.2
        name = "Ensure default seccomp profile set to docker/default or runtime/default"
        id = "CKV_K8S_32"
        supported_resources = ['kubernetes_pod_security_policy']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if "metadata" in conf:
            if "annotations" in conf["metadata"][0]:
                metadata = conf["metadata"][0]
                if metadata.get("annotations"):
                    annotations = metadata["annotations"][0]
                    if annotations is not None:
                        for annotation in annotations:
                            annotation = ''.join(annotation.split())
                            if annotation == "seccomp.security.alpha.kubernetes.io/defaultProfileName":
                                my_value = str(annotations.get(annotation))
                                if "docker/default" in my_value or "runtime/default" in my_value:
                                    return CheckResult.PASSED
        return CheckResult.FAILED


check = SeccompPSP()




from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class Ec2TransitGatewayAutoAccept(BaseResourceNegativeValueCheck):
    def __init__(self):
        """
        NIST.800-53.r5 AC-4(21), NIST.800-53.r5 CA-9(1), NIST.800-53.r5 CM-2
        EC2 Transit Gateways should not automatically accept VPC attachment requests
        """
        name = "Ensure Transit Gateways do not automatically accept VPC attachment requests"
        id = "CKV_AWS_331"
        supported_resources = ['aws_ec2_transit_gateway']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'auto_accept_shared_attachments'

    def get_forbidden_values(self):
        return "enable"


check = Ec2TransitGatewayAutoAccept()



from __future__ import annotations

from dataclasses import dataclass
from typing import Any, TYPE_CHECKING

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.checks.base_check_registry import BaseCheckRegistry

if TYPE_CHECKING:
    from checkov.common.checks.base_check import BaseCheck
    from checkov.common.typing import _SkippedCheck, _CheckResult
    from checkov.runner_filter import RunnerFilter


@dataclass
class EntityDetails:
    provider_type: str | None
    data: dict[str, Any]


class ServerlessRegistry(BaseCheckRegistry):
    def __init__(self) -> None:
        super().__init__(CheckType.SERVERLESS)

    def extract_entity_details(self, entity: EntityDetails) -> tuple[str, dict[str, Any]]:  # type:ignore[override]
        return f"serverless_{entity.provider_type}", entity.data

    def scan(  # type:ignore[override]
        self, scanned_file: str, entity: EntityDetails, skipped_checks: list[_SkippedCheck], runner_filter: RunnerFilter
    ) -> dict[BaseCheck, _CheckResult]:
        entity_type, entity_configuration = self.extract_entity_details(entity)
        results = {}
        checks = self.get_checks(entity_type)
        for check in checks:
            skip_info: _SkippedCheck = {}
            if skipped_checks:
                if check.id in [x["id"] for x in skipped_checks]:
                    skip_info = [x for x in skipped_checks if x["id"] == check.id][0]

            if runner_filter.should_run_check(check, report_type=CheckType.SERVERLESS):
                self.logger.debug(f"Running check: {check.name} on file {scanned_file}")
                result = check.run(
                    scanned_file=scanned_file,
                    entity_configuration=entity_configuration,
                    entity_name=entity_type,
                    entity_type=entity_type,
                    skip_info=skip_info,
                )
                results[check] = result
        return results



from abc import ABC, abstractmethod
from typing import List, Dict, Set, Callable, Any
import logging
import os


class AbstractAliasMappingStrategy(ABC):
    @abstractmethod
    def get_language(self) -> str:
        pass

    @abstractmethod
    def get_file_name_to_parser_map(self) -> Dict[str, Callable[[str, Set[str]], Dict[str, Any]]]:
        pass

    @staticmethod
    def _add_package_aliases(alias_mapping: Dict[str, Any], language: str, repository_name: str,
                             file_relative_path: str, package_name: str, package_aliases: List[str]) -> None:
        package_aliases_for_file = alias_mapping["languages"].setdefault(language, {"repositories": {}})["repositories"] \
            .setdefault(repository_name, {"files": {}})["files"] \
            .setdefault(file_relative_path, {"packageAliases": {}})["packageAliases"]
        if package_name in package_aliases_for_file:
            raise Exception(f"aliases for \'{package_name}\' in the file \'{file_relative_path}\' in the repository "
                            f"\'{repository_name}\' already were set")
        package_aliases_for_file[package_name] = {"packageAliases": package_aliases}

    def update_alias_mapping(self, alias_mapping: Dict[str, Any], repository_name: str, root_dir: str, relevant_packages: Set[str])\
            -> None:
        logging.debug("[AbstractAliasMappingStrategy](create_alias_mapping) - starting")
        file_name_to_parser_map = self.get_file_name_to_parser_map()
        for curr_root, _, f_names in os.walk(root_dir):
            for file_name in f_names:
                if file_name in file_name_to_parser_map:
                    logging.debug(f"[AbstractAliasMappingStrategy](create_alias_mapping) - starting parsing ${file_name}")
                    file_absolute_path = os.path.join(curr_root, file_name)
                    file_relative_path = os.path.relpath(file_absolute_path, root_dir)
                    with open(file_absolute_path) as f:
                        file_content = f.read()
                        try:
                            output = file_name_to_parser_map[file_name](file_content, relevant_packages)
                            for package_name in output["packageAliases"]:
                                self._add_package_aliases(alias_mapping, self.get_language(), repository_name,
                                                          file_relative_path, package_name,
                                                          output["packageAliases"][package_name]["packageAliases"])
                            logging.debug(
                                f"[AbstractAliasMappingStrategy](create_alias_mapping) - done parsing for ${file_name}")
                        except Exception:
                            logging.error(f"[AbstractAliasMappingStrategy](create_alias_mapping) - failure when "
                                          f"parsing the file '${file_name}'. file content:\n{file_content}.\n",
                                          exc_info=True)
                            raise



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from checkov.terraform.checks.resource.base_cloudsplaining_resource_iam_check import (
    BaseTerraformCloudsplainingResourceIAMCheck,
)

if TYPE_CHECKING:
    from cloudsplaining.scan.policy_document import PolicyDocument


class IAMStarResourcePolicyDocument(BaseTerraformCloudsplainingResourceIAMCheck):
    def __init__(self):
        name = 'Ensure no IAM policies documents allow "*" as a statement\'s resource for restrictable actions'
        id = "CKV_AWS_355"
        super().__init__(name=name, id=id)

    def cloudsplaining_analysis(self, policy: PolicyDocument) -> list[str] | list[dict[str, Any]]:
        return policy.all_allowed_unrestricted_actions


check = IAMStarResourcePolicyDocument()



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class APIGatewayMethodSettingCacheEncrypted(BaseResourceValueCheck):

    def __init__(self):
        """
        NIST.800-53.r5 CA-9(1), NIST.800-53.r5 CM-3(6), NIST.800-53.r5 SC-13, NIST.800-53.r5 SC-28,
        NIST.800-53.r5 SC-28(1), NIST.800-53.r5 SC-7(10), NIST.800-53.r5 SI-7(6)
        """
        name = "Ensure API Gateway method setting caching is set to encrypted"
        id = "CKV_AWS_308"
        supported_resources = ['aws_api_gateway_method_settings']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "settings/[0]/cache_data_encrypted"

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        settings = conf.get("settings", {})
        if settings and len(settings) == 1:
            settings = settings[0]
        cache_enabled = settings.get("caching_enabled", [False])
        if isinstance(cache_enabled, list) and len(cache_enabled) == 1:
            cache_enabled = cache_enabled[0]
        if cache_enabled:
            cache_encrypted = settings.get("cache_data_encrypted", [False])
            if isinstance(cache_encrypted, list) and len(cache_encrypted) == 1:
                cache_encrypted = cache_encrypted[0]
            if not cache_encrypted:
                return CheckResult.FAILED
        return CheckResult.PASSED


check = APIGatewayMethodSettingCacheEncrypted()



from __future__ import annotations

import logging
from enum import Enum
from typing import Dict, Any

from checkov.common.bridgecrew.severities import Severities

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.models.enums import CheckResult
from checkov.common.output.record import Record
from checkov.common.output.report import Report

from checkov.common.runners.base_post_runner import BasePostRunner
from checkov.common.util.type_forcers import force_list
from checkov.policies_3d.checks_parser import Policy3dParser
from checkov.policies_3d.record import Policy3dRecord
from checkov.policies_3d.checks_infra.base_check import Base3dPolicyCheck
from checkov.policies_3d.syntax.iac_syntax import IACPredicate
from checkov.policies_3d.syntax.cves_syntax import CVEPredicate
from checkov.policies_3d.syntax.secrets_syntax import SecretsPredicate
from checkov.runner_filter import RunnerFilter


class CVECheckAttribute(str, Enum):
    RISK_FACTORS = "risk_factor"


class CVEReportAttribute(str, Enum):
    RISK_FACTORS = 'riskFactors'


CVE_CHECK_TO_REPORT_ATTRIBUTE = {
    CVECheckAttribute.RISK_FACTORS: CVEReportAttribute.RISK_FACTORS
}

module_to_check_types = {
    "iac": filter(lambda _type: _type in [CheckType.SECRETS, CheckType.SCA_IMAGE], list(CheckType.__dict__.keys())),
    "secrets": [CheckType.SECRETS],
    "cves": [CheckType.SCA_IMAGE]
}


class Policy3dRunner(BasePostRunner):
    check_type = CheckType.POLICY_3D  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        super().__init__()

    def run_v2(self,
               raw_checks: list[dict[str, Any]] | None = None,
               scan_reports: list[Report] | None = None,
               runner_filter: RunnerFilter | None = None
               ) -> Report:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        report = Report(self.check_type)

        if not raw_checks or not scan_reports:
            logging.debug("No checks or reports scan, skipping 3D policies runner")
            return report

        self.pbar.initiate(len(raw_checks))

        failed_checks_by_resource = self.create_failed_checks_by_resource_mapping(scan_reports)
        records_3d = []
        for raw_3d_check in raw_checks:
            for resource, modules_records in failed_checks_by_resource.items():
                iac_records: list[Record] = modules_records.get("iac", [])
                secrets_records: list[Record] = modules_records.get("secrets", [])
                cves_reports: list[dict[str, Any]] = modules_records.get("cves", [])
                check = Policy3dParser(raw_3d_check).parse(
                    iac_records=iac_records,
                    secrets_records=secrets_records,
                    cves_reports=cves_reports
                )

                if not check:
                    continue

                check.severity = Severities[raw_3d_check['severity']]
                check.bc_id = check.id

                check_result = CheckResult.PASSED
                if any([predicament() for predicament in check.predicaments]):
                    check_result = CheckResult.FAILED
                    logging.debug(f"Resource {resource} is violating 3D policy {check.bc_id}")

                records_3d.append(self.create_record(check, check_result))

        for record in records_3d:
            if record:
                report.add_record(record=record)

        return report

    @staticmethod
    def create_record(check: Base3dPolicyCheck, check_result: CheckResult) -> Record | None:
        true_vulnerabilities_cve_reports = []
        true_iac_records = []
        true_secrets_records = []

        records_ids = set()

        any_record_data_source = None
        record_data_source: Record | None

        all_predicates = set()
        for predicament in check.predicaments:
            all_predicates.update(predicament.get_all_children_predicates())

        for predicate in all_predicates:
            if not any_record_data_source and (isinstance(predicate, IACPredicate) or isinstance(predicate, SecretsPredicate)):
                any_record_data_source = predicate.record

            if predicate.is_true:
                if isinstance(predicate, IACPredicate) and predicate.record.bc_check_id not in records_ids:
                    true_iac_records.append(predicate.record)
                    records_ids.add(predicate.record.bc_check_id)
                elif isinstance(predicate, CVEPredicate) and predicate.cve_report.get('cveId') not in records_ids:
                    true_vulnerabilities_cve_reports.append(predicate.cve_report)
                    records_ids.add(predicate.cve_report.get('cveId'))
                elif isinstance(predicate, SecretsPredicate) and predicate.record.bc_check_id not in records_ids:
                    true_secrets_records.append(predicate.record)
                    records_ids.add(predicate.record.bc_check_id)
        try:
            record_data_source = list(true_iac_records)[0] if len(true_iac_records) > 0 else list(true_secrets_records)[0]
        except IndexError:
            record_data_source = any_record_data_source

        if not record_data_source:
            logging.debug("Unable to create record from an empty record data source")
            return None

        record = Policy3dRecord(
            check_id=check.id,
            bc_check_id=check.bc_id,
            check_name=check.name,
            check_result={'result': check_result},
            code_block=record_data_source.code_block,
            file_path=record_data_source.file_path,
            file_line_range=record_data_source.file_line_range,
            resource=f'{record_data_source.file_path}:{record_data_source.resource}',
            evaluations=None,
            check_class=check.__class__.__module__,
            file_abs_path=record_data_source.file_abs_path,
            severity=check.severity,
            vulnerabilities=true_vulnerabilities_cve_reports,
            iac_records=true_iac_records + true_secrets_records,
            composed_from_iac_records=true_iac_records,
            composed_from_secrets_records=true_secrets_records,
            composed_from_cves=true_vulnerabilities_cve_reports
        )

        record.set_guideline(check.guideline)
        return record

    @staticmethod
    def create_failed_checks_by_resource_mapping(scan_reports: list[Report]) -> dict[str, Any]:
        """
        Output structure:
        {
            resource_id: {
                "iac": [...], # list of failed checks of type Record
                "secrets": [...], # list of failed checks of type Record
                "cves": [...] # list of vulnerabilities of type ReportCVE
            }
        }
        """
        failed_checks_by_resource: dict[str, Any] = {}
        for report in scan_reports:
            if report.check_type == CheckType.SCA_IMAGE:
                # Save image cached results on a resource
                for result in report.image_cached_results:
                    resource_id = result.get('relatedResourceId', '').split(result.get('dockerFilePath'))[1][1:]
                    if resource_id in failed_checks_by_resource.keys():
                        if "cves" not in failed_checks_by_resource[resource_id]:
                            failed_checks_by_resource[resource_id]["cves"] = []
                        failed_checks_by_resource[resource_id]["cves"] += result.get("vulnerabilities", [])
                    else:
                        failed_checks_by_resource[resource_id] = {}
                        failed_checks_by_resource[resource_id]["cves"] = result.get("vulnerabilities", [])

                    for cve in failed_checks_by_resource[resource_id]["cves"]:
                        cve["dockerImageName"] = result.get("dockerImageName")

            else:
                # Save failed checks on a resource
                iac_or_secrets = "secrets" if report.check_type == CheckType.SECRETS else "iac"
                for failed_check in report.failed_checks:
                    resource = failed_check.resource.lstrip(failed_check.file_abs_path)
                    if failed_check.resource in failed_checks_by_resource.keys():
                        failed_checks_by_resource[resource][iac_or_secrets].append(failed_check)
                    else:
                        failed_checks_by_resource[resource] = {}
                        failed_checks_by_resource[resource][iac_or_secrets] = [failed_check]

        return failed_checks_by_resource

    def run(  # type:ignore[override]
            self,
            checks: list[Base3dPolicyCheck] | None = None,
            scan_reports: list[Report] | None = None,
            runner_filter: RunnerFilter | None = None
    ) -> Report:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        report = Report(self.check_type)

        if not checks or not scan_reports:
            logging.debug("No checks or reports scan.")
            return report

        self.pbar.initiate(len(checks))

        reports_by_framework = {report.check_type: report for report in scan_reports}
        for check in checks:
            records = self.collect_check(check, reports_by_framework)
            for record in records:
                report.add_record(record=record)

        self.pbar.close()
        return report

    def collect_check(self, check: Base3dPolicyCheck, reports_by_fw: dict[str, Report]) -> list[Policy3dRecord]:
        records = []
        iac_results_map = self.solve_check_iac(check, reports_by_fw)
        cve_results_map = self.solve_check_cve(check, reports_by_fw)

        for iac_resource, iac_records in iac_results_map.items():
            for cve_resource, vulnerabilities in cve_results_map.items():
                if iac_resource == cve_resource:
                    # This means we found the combination on the same resource -> create a violation for that resource
                    check_result = CheckResult.FAILED
                    iac_records = [record for record in iac_records]
                    record = self.get_record(check, iac_records[0], vulnerabilities, check_result, iac_records)
                    record.set_guideline(check.guideline)
                    records.append(record)

        self.pbar.update()
        return records

    def solve_check_iac(self, check: Base3dPolicyCheck, reports_by_fw: dict[str, Report]) -> dict[str, list[Record]]:
        iac_results_map: dict[str, list[Record]] = {}
        if check.iac:
            for framework, bc_check_ids in check.iac.items():
                framework_report = reports_by_fw.get(framework)
                if framework_report:
                    fw_records = framework_report.failed_checks
                    for record in fw_records:
                        if record.bc_check_id in bc_check_ids:
                            resource_id = f'{record.file_abs_path}:{record.resource}'
                            if resource_id in iac_results_map:
                                iac_results_map[resource_id].append(record)
                            else:
                                iac_results_map[resource_id] = [record]

                    # the following implements the AND logic for the list of iac expected check ids
                    for resource in list(iac_results_map.keys()):
                        if len(iac_results_map[resource]) != len(bc_check_ids):
                            del iac_results_map[resource]

        return iac_results_map

    def solve_check_cve(self, check: Base3dPolicyCheck, reports_by_fw: dict[str, Report]) -> dict[str, list[Dict[str, Any]]]:
        cve_results_map: dict[str, list[Dict[str, Any]]] = {}
        if check.cve:
            cve_report = reports_by_fw.get(CheckType.SCA_IMAGE)
            if cve_report:
                image_results = cve_report.image_cached_results
                for attribute, value in check.cve.items():
                    for image_result in image_results:
                        matching_cves = [vuln for vuln in image_result.get('vulnerabilities', []) if
                                         value[0] in force_list(vuln.get(CVE_CHECK_TO_REPORT_ATTRIBUTE[attribute], []))]
                        if matching_cves:
                            image_related_resource = image_result.get('relatedResourceId')
                            image_name = image_result.get('dockerImageName')
                            if not image_related_resource or not image_name:
                                logging.debug(
                                    "[policies3d/runner](solve_check_cve) Found vulnerabilities of an image without a related resource or image name, skipping")
                                break
                            for cve in matching_cves:
                                cve['dockerImageName'] = image_name
                            # The current logic for multiple cve conditions in the policy is of "OR" - we add all of
                            # the matching cves, even if matched only by a single policy attribute. To implement an
                            # "AND" logic for the combination of conditions, the matching cves need to be filtered
                            # before being added to the result.
                            if image_related_resource in cve_results_map:
                                cve_results_map[image_related_resource].extend(matching_cves)
                            else:
                                cve_results_map[image_related_resource] = matching_cves
        return cve_results_map

    def get_record(self, check: Base3dPolicyCheck, iac_record: Record, vulnerabilities: list[Dict[str, Any]],
                   check_result: CheckResult, iac_records: list[Record]) -> Policy3dRecord:
        return Policy3dRecord(
            check_id=check.id,
            bc_check_id=check.bc_id,
            check_name=check.name,
            check_result={'result': check_result},
            code_block=iac_record.code_block,
            file_path=iac_record.file_path,
            file_line_range=iac_record.file_line_range,
            resource=f'{iac_record.file_path}:{iac_record.resource}',
            evaluations=None,
            check_class=check.__class__.__module__,
            file_abs_path=iac_record.file_abs_path,
            severity=check.severity,
            vulnerabilities=vulnerabilities,
            iac_records=iac_records,
            composed_from_iac_records=[],
            composed_from_secrets_records=[],
            composed_from_cves=[]
        )



from json import JSONDecodeError
import re

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list
import json
from typing import List


def check_conditions(statement) -> bool:
    # Check if 'Condition' key exists
    if 'Condition' not in statement:
        return False

    condition = statement['Condition']

    # Pass if they define bad ARNs. Assumes they are not too narrow
    if any(key in condition for key in ['ArnNotEquals', 'ArnNotLike']):
        return True

    # Handling 'ArnEquals' and 'ArnLike'
    for arn_key in ['ArnEquals', 'ArnLike']:
        if arn_key in condition:
            # Pass unless it is for all IAM ARNs
            for principal_key in ['aws:PrincipalArn', 'aws:SourceArn']:
                if principal_key in condition[arn_key]:
                    principal_arn = condition[arn_key][principal_key]
                    # Fail if the  Condition is for all ARNs of any resource
                    if re.match(r'^arn:aws:[a-z0-9-]+::\*.*$', principal_arn):
                        return False
            # Passed if 'aws:PrincipalArn' or 'aws:SourceArn' do not match because then they are specific
            return True

    # Handle VPC sources. Other sources not specific enough
    # Leaves out the NOT conditions as too broad ('StringNotEquals', 'StringNotEqualsIgnoreCase', 'StringNotLike')
    string_conditions = ['StringEquals', 'StringEqualsIgnoreCase', 'StringLike']
    if any(condition_type in condition for condition_type in string_conditions):
        for condition_type in string_conditions:
            if condition_type in condition:
                if any(source in condition[condition_type] for source in ['aws:sourceVpce', 'aws:SourceVpc',
                                                                          'aws:PrincipalOrgPath', 'aws:userid']):
                    return True

    # Default fail if none of the above conditions are met
    return False


class S3AllowsAnyPrincipal(BaseResourceCheck):

    def __init__(self) -> None:
        name = "Ensure S3 bucket does not allow an action with any Principal"
        id = "CKV_AWS_70"
        supported_resources = ['aws_s3_bucket', 'aws_s3_bucket_policy']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'policy' not in conf.keys():
            return CheckResult.UNKNOWN
        if not isinstance(conf['policy'][0], str):
            policy_block = conf['policy'][0]
        else:
            if "data.aws_iam_policy_document" in conf['policy'][0]:
                return CheckResult.UNKNOWN
            else:
                try:
                    policy_block = json.loads(conf['policy'][0])
                except JSONDecodeError:  # nosec
                    return CheckResult.UNKNOWN

        if isinstance(policy_block, dict) and 'Statement' in policy_block.keys():
            for statement in force_list(policy_block['Statement']):
                if statement['Effect'] == 'Deny' or 'Principal' not in statement:
                    continue
                principal = statement['Principal']
                if principal == '*':
                    if check_conditions(statement):
                        return CheckResult.PASSED
                    return CheckResult.FAILED
                if 'AWS' in statement['Principal']:
                    # Can be a string or an array of strings
                    aws = statement['Principal']['AWS']
                    if (isinstance(aws, str) and aws == '*') or (isinstance(aws, list) and '*' in aws):
                        if check_conditions(statement):
                            return CheckResult.PASSED
                        return CheckResult.FAILED
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ['policy']


check = S3AllowsAnyPrincipal()



from __future__ import annotations

from datetime import timedelta
import logging
from functools import wraps
from timeit import default_timer
from typing import TypeVar, Callable

from typing_extensions import ParamSpec

T = TypeVar("T")
P = ParamSpec("P")


def time_it(func: Callable[P, T]) -> Callable[P, T]:
    """Prints the time it took to execute the function"""

    @wraps(func)
    def wrapper(*args: P.args, **kwargs: P.kwargs) -> T:
        start = default_timer()
        output = func(*args, **kwargs)
        end = default_timer()

        func_path = f"{func.__code__.co_filename.replace('.py', '')}.{func.__name__}"
        logging.info(f"'{func_path}' took: {timedelta(seconds=end - start)}\n")

        return output
    return wrapper



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list


class S3ProtectAgainstPolicyLockout(BaseResourceCheck):

    def __init__(self) -> None:
        name = "Ensure S3 bucket policy does not lockout all but root user. (Prevent lockouts needing root account fixes)"
        id = "CKV_AWS_93"
        supported_resources = ('aws_s3_bucket', 'aws_s3_bucket_policy')
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        fails = ["s3:PutBucketPolicy", "s3:*BucketPolicy", "s3:Put*", "s3:*", "*"]

        if 'policy' not in conf.keys() or not isinstance(conf['policy'][0], dict):
            return CheckResult.PASSED
        try:
            policy_block = conf.get('policy')[0]
            if 'Statement' in policy_block.keys():
                for statement in force_list(policy_block['Statement']):
                    if 'Condition' in statement.keys() or 'NotAction' in statement.keys() \
                            or statement.get('Effect') != 'Deny':
                        # https://github.com/bridgecrewio/checkov/pull/627#issuecomment-714681751
                        continue

                    principal = statement['Principal']
                    if 'AWS' in statement['Principal']:
                        # Can be a string or an array of strings
                        aws = statement['Principal']['AWS']
                        if (isinstance(aws, str) and aws == '*') or (isinstance(aws, list) and '*' in aws):
                            principal = "*"

                    action = statement['Action']
                    if principal == "*":
                        if action == '*':
                            return CheckResult.FAILED
                        myActions = force_list(action)
                        for item in myActions:
                            if item in fails:
                                return CheckResult.FAILED
        except Exception:  # nosec
            pass
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> list[str]:
        return ['policy']


check = S3ProtectAgainstPolicyLockout()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class WinVMEncryptionAtHost(BaseResourceValueCheck):
    def __init__(self):
        """
        If enabled, all the disks (including the temp disk) attached to this Virtual Machine will be encrypted

        if not enabled:
        https://learn.microsoft.com/en-gb/azure/virtual-machines/disks-enable-host-based-encryption-portal?tabs=azure-cli#prerequisites
        """
        name = "Ensure Windows VM enables encryption"
        id = "CKV_AZURE_151"
        supported_resources = ['azurerm_windows_virtual_machine']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "encryption_at_host_enabled"


check = WinVMEncryptionAtHost()



from __future__ import annotations

import json
import logging
from typing import TYPE_CHECKING, Any, cast

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.checks_infra.registry import get_graph_checks_registry
from checkov.common.bridgecrew.integration_features.base_integration_feature import BaseIntegrationFeature
from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.bridgecrew.severities import Severities, get_severity
from checkov.common.checks.base_check_registry import BaseCheckRegistry

if TYPE_CHECKING:
    from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
    from checkov.common.bridgecrew.severities import Severity
    from checkov.common.output.report import Report
    from checkov.common.typing import _BaseRunner


class PolicyMetadataIntegration(BaseIntegrationFeature):
    def __init__(self, bc_integration: BcPlatformIntegration) -> None:
        super().__init__(bc_integration=bc_integration, order=0)
        self.check_metadata: dict[str, Any] = {}
        self.sast_check_metadata: dict[str, Any] = {}
        self.bc_to_ckv_id_mapping: dict[str, str] = {}
        self.pc_to_ckv_id_mapping: dict[str, str] = {}
        self.ckv_id_to_source_incident_id_mapping: dict[str, str] = {}
        self.severity_key = 'severity'
        self.filtered_policy_ids: list[str] = []
        self.filtered_exception_policy_ids: list[str] = []

    def is_valid(self) -> bool:
        return (
            not self.bc_integration.skip_download
            and not self.integration_feature_failures
        )

    def pre_scan(self) -> None:
        try:
            if self.bc_integration.customer_run_config_response:
                self._handle_customer_run_config(self.bc_integration.customer_run_config_response)
                if self.bc_integration.is_prisma_integration():
                    # build a list of policy ids included using the --prisma-metadata-filter flag
                    self._handle_customer_prisma_policy_metadata(self.bc_integration.prisma_policies_response, exclude_policies=False)
                    # build a list of policy ids excluded using the --prisma-metadata-filter-exception flag
                    self._handle_customer_prisma_policy_metadata(self.bc_integration.prisma_policies_exception_response, exclude_policies=True)
            elif self.bc_integration.public_metadata_response:
                self._handle_public_metadata(self.bc_integration.public_metadata_response)
            else:
                logging.debug('In the pre-scan for policy metadata, but nothing was fetched from the platform')
                self.integration_feature_failures = True
                return

            all_checks = BaseCheckRegistry.get_all_registered_checks()

            if self.config and self.config.framework and "all" not in self.config.framework:
                registries = self.config.framework
                if CheckType.TERRAFORM_PLAN in registries and CheckType.TERRAFORM not in registries:
                    registries.append(CheckType.TERRAFORM)
            else:
                registries = (CheckType.TERRAFORM, CheckType.CLOUDFORMATION, CheckType.KUBERNETES, CheckType.BICEP, CheckType.TERRAFORM_PLAN)

            for r in registries:
                registry = get_graph_checks_registry(r)
                registry.load_checks()
                all_checks += registry.checks

            use_prisma_metadata = self.bc_integration.is_prisma_integration()

            if use_prisma_metadata:
                self.severity_key = 'pcSeverity'

            for check in all_checks:
                checkov_id = check.id
                metadata = self.get_policy_metadata(checkov_id)
                if metadata:
                    check.bc_id = metadata.get('id')
                    check.guideline = metadata.get('guideline')

                    # fall back on plain severity if there is no PC severity
                    check.severity = get_severity(metadata.get(self.severity_key, metadata.get('severity')))
                    check.bc_category = metadata.get('category')
                    check.benchmarks = metadata.get('benchmarks')

                    if use_prisma_metadata and metadata.get('descriptiveTitle'):
                        check.name = metadata['descriptiveTitle']
                else:
                    check.bc_id = None
        except Exception:
            self.integration_feature_failures = True
            logging.debug('An error occurred loading policy metadata. Some metadata may be missing from the run.', exc_info=True)

    def get_bc_id(self, checkov_id: str) -> str:
        return cast(str, self.check_metadata.get(checkov_id, {}).get('id'))

    def get_guideline(self, checkov_id: str) -> str:
        return cast(str, self.check_metadata.get(checkov_id, {}).get('guideline'))

    def get_severity(self, checkov_id: str) -> Severity | None:
        severity: str | Severity | None = self.check_metadata.get(checkov_id, {}).get(self.severity_key)
        if not severity:
            severity = self.check_metadata.get(checkov_id, {}).get('severity')
        if severity and isinstance(severity, str):
            return Severities[severity]  # not all runners register their checks in time for being processed above
        return cast(None, severity)

    def get_category(self, checkov_id: str) -> str:
        return cast(str, self.check_metadata.get(checkov_id, {}).get('category'))

    def get_benchmarks(self, checkov_id: str) -> dict[str, list[str]] | None:
        return cast("dict[str, list[str]] | None", self.check_metadata.get(checkov_id, {}).get('benchmarks'))

    def get_prisma_policy_title(self, checkov_id: str) -> str:
        return cast(str, self.check_metadata.get(checkov_id, {}).get('descriptiveTitle'))

    def get_policy_metadata(self, checkov_id: str) -> dict[str, Any] | None:
        return self.check_metadata.get(checkov_id)

    def get_ckv_id_from_bc_id(self, bc_id: str) -> str | None:
        return self.bc_to_ckv_id_mapping.get(bc_id)

    def get_ckv_id_from_pc_id(self, pc_id: str) -> str | None:
        return self.pc_to_ckv_id_mapping.get(pc_id)

    def get_source_incident_id_from_ckv_id(self, ckv_id: str) -> str | None:
        return self.ckv_id_to_source_incident_id_mapping.get(ckv_id)

    def _handle_public_metadata(self, check_metadata: dict[str, Any]) -> None:
        guidelines = check_metadata['guidelines']
        self.bc_to_ckv_id_mapping = check_metadata['idMapping']

        for ckv_id, guideline in guidelines.items():
            self.check_metadata[ckv_id] = {
                'guideline': guideline
            }

        for bc_id, ckv_id in self.bc_to_ckv_id_mapping.items():
            if ckv_id in self.check_metadata:
                self.check_metadata[ckv_id]['id'] = bc_id
            else:
                self.check_metadata[ckv_id] = {
                    'id': bc_id
                }

    def _handle_customer_run_config(self, run_config: dict[str, Any]) -> None:
        self.check_metadata = run_config['policyMetadata']
        for ckv_id, pol in self.check_metadata.items():
            if 'SAST' in ckv_id:
                self.sast_check_metadata[ckv_id] = pol
            self.bc_to_ckv_id_mapping[pol['id']] = ckv_id
            if self.bc_integration.is_prisma_integration() and pol.get('pcPolicyId'):
                self.pc_to_ckv_id_mapping[pol['pcPolicyId']] = ckv_id
        # Custom policies are returned in run_config['customPolicies'] rather than run_config['policyMetadata'].
        if 'customPolicies' in run_config:
            for custom_policy in run_config['customPolicies']:
                custom_policy_check_metadata = {
                    'severity': custom_policy.get('severity'),
                    'guideline': custom_policy.get('guideline')
                }
                self.check_metadata[custom_policy['id']] = {k: v for k, v in custom_policy_check_metadata.items() if v is not None}
                pc_policy_id = custom_policy.get('pcPolicyId')
                if pc_policy_id:
                    self.pc_to_ckv_id_mapping[pc_policy_id] = custom_policy['id']
                source_incident_id = custom_policy.get('sourceIncidentId')
                if source_incident_id:
                    self.ckv_id_to_source_incident_id_mapping[custom_policy['id']] = source_incident_id

    def _handle_customer_prisma_policy_metadata(self, prisma_policy_metadata: list[dict[str, Any]], exclude_policies: bool) -> None:
        policy_ids = list()
        if isinstance(prisma_policy_metadata, list):
            for metadata in prisma_policy_metadata:
                logging.debug(f"Parsing filtered_policy_ids from metadata: {json.dumps(metadata)}")
                pc_id = metadata.get('policyId')
                if pc_id:
                    ckv_id = self.get_ckv_id_from_pc_id(pc_id)
                    if ckv_id:
                        policy_ids.append(ckv_id)
            if exclude_policies:
                self.filtered_exception_policy_ids = policy_ids
                self._add_ckv_id_for_filtered_cloned_checks(self.filtered_exception_policy_ids, exclude_policies)
            else:
                self.filtered_policy_ids = policy_ids
                self._add_ckv_id_for_filtered_cloned_checks(self.filtered_policy_ids, exclude_policies)

    def _add_ckv_id_for_filtered_cloned_checks(self, policy_ids: list[str], exclude_policies: bool) -> None:
        """
        Filtered checks are the policies that are returned by --policy-metadata-filter.
        Filtered exclusion checks are the policies that are returned by --policy-metadata-filter-exclusion.
        Cloned checks are policies that have modified metadata in Prisma (severity, title etc).
        Filtered checks do not have a definition if they are cloned, instead they have a sourceIncidentId
        which corresponds to the BC ID of the original source check.
        This method adds the CKV ID for that source check to the list of filtered policies to ensure it is run.
        Example:
            Input:
                filtered_policy_ids = [ "org_AWS_1609123441" ]
                ckv_id_to_source_incident_id_mapping =  { "org_AWS_1609123441": "BC_AWS_GENERAL_123" }
                bc_id_to_ckv_id_mapping = { "BC_AWS_GENERAL_123": "CKV_AWS_123" }
            Output:
                filtered_policy_ids = [ "org_AWS_1609123441", "CKV_AWS_123" ]
        """
        ckv_ids = []
        for policy_id in policy_ids:
            source_bc_id = self.get_source_incident_id_from_ckv_id(policy_id)
            if not source_bc_id:
                continue
            ckv_id = self.get_ckv_id_from_bc_id(source_bc_id)
            if not ckv_id:
                continue
            ckv_ids.append(ckv_id)
        if exclude_policies:
            self.filtered_exception_policy_ids += ckv_ids
            return
        self.filtered_policy_ids += ckv_ids

    def pre_runner(self, runner: _BaseRunner) -> None:
        # not used
        pass

    def post_runner(self, scan_reports: Report) -> None:
        # not used
        pass

    def post_scan(self, merged_reports: list[Report]) -> None:
        # not used
        pass


integration = PolicyMetadataIntegration(bc_integration)



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class StorageAccountDisablePublicAccess(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Storage accounts disallow public access"
        id = "CKV_AZURE_59"
        supported_resources = ("azurerm_storage_account",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def get_inspected_key(self) -> str:
        return "public_network_access_enabled"

    def get_expected_values(self) -> list[Any]:
        return [False]


check = StorageAccountDisablePublicAccess()



from typing import Dict, Any, List, Union
from pydantic import BaseModel, create_model

from checkov.common.sast.consts import SastLanguages


class SastPolicyMetadataEntry(BaseModel):
    ID: str  # noqa: CCE003
    Name: str  # noqa: CCE003
    Guidelines: str  # noqa: CCE003
    Category: str  # noqa: CCE003
    Severity: str  # noqa: CCE003
    CWE: List[str]  # noqa: CCE003
    OWASP: Union[List[str], None]  # noqa: CCE003


class SastPolicyEntry(BaseModel):
    Metadata: SastPolicyMetadataEntry  # noqa: CCE003
    Language: SastLanguages  # noqa: CCE003
    Definition: Dict[str, Any]  # noqa: CCE003


# dynamically typing the object of SastPolicies
fields = {lang.value: (List[SastPolicyEntry], []) for lang in SastLanguages}  # type: ignore
SastPolicies = create_model('SastPolicies', **fields)  # type: ignore



from typing import Any, List, Dict

from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck

# flow logs can't be enabled for subnetworks with the following purpose set
PURPOSE_EXCEPTIONS = ["INTERNAL_HTTPS_LOAD_BALANCER", "REGIONAL_MANAGED_PROXY", "GLOBAL_MANAGED_PROXY"]


class GoogleSubnetworkLoggingEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that VPC Flow Logs is enabled for every subnet in a VPC Network"
        id = "CKV_GCP_26"
        supported_resources = ("google_compute_subnetwork",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        purpose = conf.get("purpose")
        if purpose and purpose[0] in PURPOSE_EXCEPTIONS:
            return CheckResult.UNKNOWN

        return super().scan_resource_conf(conf)

    def get_inspected_key(self) -> str:
        return "log_config"

    def get_expected_values(self) -> List[Any]:
        return [ANY_VALUE]


check = GoogleSubnetworkLoggingEnabled()



from __future__ import annotations

from typing import Any

from checkov.serverless.checks.function.base_function_check import BaseFunctionCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.serverless.parsers.parser import IAM_ROLE_STATEMENTS_TOKEN


class StarActionPolicyDocument(BaseFunctionCheck):
    def __init__(self) -> None:
        name = "Ensure no IAM policies documents allow \"*\" as a statement's actions"
        id = "CKV_AWS_49"
        supported_entities = ('serverless_aws',)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_entities)

    def scan_function_conf(self, conf: dict[str, Any]) -> CheckResult:
        """
        validates iam policy document
         https://learn.hashicorp.com/terraform/aws/iam-policy
        :param conf: aws_kms_key configuration
        :return: <CheckResult>
        """
        key = IAM_ROLE_STATEMENTS_TOKEN
        statements = conf.get(key)
        if not statements:
            return CheckResult.PASSED
        for statement in statements:
            if not isinstance(statement, dict):
                return CheckResult.UNKNOWN
            if 'Action' in statement and '*' in statement['Action'] and statement.get('Effect') == 'Allow':
                return CheckResult.FAILED
        return CheckResult.PASSED


check = StarActionPolicyDocument()



from __future__ import annotations

from typing import Any

from checkov.argo_workflows.checks.base_argo_workflows_check import BaseArgoWorkflowsCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.yaml_doc.enums import BlockType


class RunAsNonRoot(BaseArgoWorkflowsCheck):
    def __init__(self) -> None:
        name = "Ensure Workflow pods are running as non-root user"
        id = "CKV_ARGO_2"
        super().__init__(
            name=name,
            id=id,
            categories=(CheckCategories.IAM,),
            supported_entities=("spec",),
            block_type=BlockType.OBJECT,
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        security_context = conf.get("securityContext")

        if isinstance(security_context, dict) and security_context.get("runAsNonRoot") is True:
            return CheckResult.PASSED, conf

        return CheckResult.FAILED, conf


check = RunAsNonRoot()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class CloudsearchDomainEnforceHttps(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure AWS Cognito identity pool does not allow unauthenticated guest access"
        id = "CKV_AWS_366"
        supported_resources = ["aws_cognito_identity_pool"]
        categories = [CheckCategories.IAM]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def get_inspected_key(self) -> str:
        return "allow_unauthenticated_identities"

    def get_expected_value(self) -> bool:
        return False


check = CloudsearchDomainEnforceHttps()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


class StorageAccountMinimumTlsVersion(BaseResourceCheck):
    def __init__(self) -> None:
        """
            Looks for min_tls_version configuration at azurerm_storage_account to be set to TLS1_2
            https://www.terraform.io/docs/providers/azurerm/r/storage_account.html#min_tls_version
            :param conf: azurerm_storage_account configuration
            :return: <CheckResult>
        """
        name = "Ensure Storage Account is using the latest version of TLS encryption"
        id = "CKV_AZURE_44"
        supported_resources = ('Microsoft.Storage/storageAccounts',)
        categories = (CheckCategories.NETWORKING,)

        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        if "properties" in conf and \
            "minimumTlsVersion" in conf["properties"] and \
                conf["properties"]["minimumTlsVersion"] in ['TLS1_2']:
            return CheckResult.PASSED
        return CheckResult.FAILED


check = StorageAccountMinimumTlsVersion()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class SagemakerNotebookInstanceAllowsIMDSv2(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Amazon SageMaker Notebook Instance only allows for IMDSv2"
        id = "CKV_AWS_371"
        supported_resources = ['aws_sagemaker_notebook_instance']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'instance_metadata_service_configuration/[0]/minimum_instance_metadata_service_version'

    def get_expected_value(self):
        return "2"


check = SagemakerNotebookInstanceAllowsIMDSv2()



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class CodePipelineArtifactsEncrypted(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure CodePipeline Artifact store is using a KMS CMK"
        id = "CKV_AWS_219"
        supported_resources = ("aws_codepipeline",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "artifact_store/[0]/encryption_key/[0]/id"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = CodePipelineArtifactsEncrypted()



from typing import Any

from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class DocDBAuditLogs(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure DocumentDB has audit logs enabled"
        id = "CKV_AWS_104"
        supported_resources = ["AWS::DocDB::DBClusterParameterGroup"]
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "Properties/Parameters/audit_logs"

    def get_expected_value(self) -> Any:
        return "enabled"


check = DocDBAuditLogs()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


class AKSLoggingEnabled(BaseResourceCheck):
    def __init__(self) -> None:
        # apiVersion 2017-08-03 = Fail - No addonProfiles option to configure
        name = "Ensure AKS logging to Azure Monitoring is Configured"
        id = "CKV_AZURE_4"
        supported_resources = ("Microsoft.ContainerService/managedClusters",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        if "apiVersion" in conf:
            if conf["apiVersion"] == "2017-08-31":
                # No addonProfiles option to configure
                return CheckResult.FAILED

        properties = conf.get("properties")
        if isinstance(properties, dict):
            addon_profiles = properties.get("addonProfiles")
            if isinstance(addon_profiles, dict):
                omsagent = addon_profiles.get("omsagent")
                if not omsagent:
                    # it can be written in lowercase or camelCase
                    omsagent = addon_profiles.get("omsAgent")

                if isinstance(omsagent, dict) and omsagent.get("enabled"):
                    return CheckResult.PASSED

        return CheckResult.FAILED


check = AKSLoggingEnabled()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class MSSQLServerMinTLSVersion(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure MSSQL is using the latest version of TLS encryption"
        id = "CKV_AZURE_52"
        supported_resources = ("Microsoft.Sql/servers",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name,
                         id=id,
                         categories=categories,
                         supported_resources=supported_resources,
                         missing_block_result=CheckResult.FAILED,)

    def get_inspected_key(self) -> str:
        return "properties/minimalTlsVersion"

    def get_expected_value(self) -> str:
        return "1.2"


check = MSSQLServerMinTLSVersion()



from __future__ import annotations

from typing import TYPE_CHECKING, Any, cast

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.checks.base_check_registry import BaseCheckRegistry
from checkov.common.models.enums import CheckResult

if TYPE_CHECKING:
    from dockerfile_parse.parser import _Instruction  # only in extra_stubs
    from checkov.common.checks.base_check import BaseCheck
    from checkov.common.typing import _SkippedCheck, _CheckResult
    from checkov.runner_filter import RunnerFilter


class Registry(BaseCheckRegistry):
    def __init__(self) -> None:
        super().__init__(CheckType.DOCKERFILE)

    def extract_entity_details(self, entity: dict[str, Any]) -> tuple[str, str, dict[str, Any]]:
        # not needed
        return "", "", {}

    def scan(
        self,
        scanned_file: str,
        entity: dict[str, list[_Instruction]],
        skipped_checks: list[_SkippedCheck],
        runner_filter: RunnerFilter,
        report_type: str | None = None,
    ) -> dict[BaseCheck, _CheckResult]:

        results: "dict[BaseCheck, _CheckResult]" = {}
        if not entity:
            return results

        skipped_check_ids = {skipped_check["id"]: skipped_check for skipped_check in skipped_checks}

        for instruction, checks in self.checks.items():
            if instruction in entity:
                for check in checks:
                    skip_info: "_SkippedCheck" = {}
                    if skipped_check_ids:
                        if check.id in skipped_check_ids:
                            skip_info = skipped_check_ids[check.id]

                    if runner_filter.should_run_check(check, report_type=CheckType.DOCKERFILE):
                        self.update_result(
                            check=check,
                            entity_configuration=entity[instruction],
                            entity_name=instruction,
                            entity_type=instruction,
                            results=results,
                            scanned_file=scanned_file,
                            skip_info=skip_info,
                        )

        for check in self.wildcard_checks["*"]:
            skip_info = {}
            if skipped_check_ids:
                if check.id in skipped_check_ids:
                    skip_info = skipped_check_ids[check.id]

            if runner_filter.should_run_check(check, report_type=CheckType.DOCKERFILE):
                self.update_result(
                    check=check,
                    entity_configuration=entity,
                    entity_name=scanned_file,
                    entity_type="*",
                    results=results,
                    scanned_file=scanned_file,
                    skip_info=skip_info,
                )
        return results

    def update_result(
        self,
        check: BaseCheck,
        entity_configuration: list[_Instruction] | dict[str, list[_Instruction]],
        entity_name: str,
        entity_type: str,
        results: dict[BaseCheck, _CheckResult],
        scanned_file: str,
        skip_info: _SkippedCheck
    ) -> None:
        result = self.run_check(
            check=check,
            entity_configuration=entity_configuration,  # type:ignore[arg-type]  # special Dockerfile runner behaviour
            entity_name=entity_name,
            entity_type=entity_type,
            scanned_file=scanned_file,
            skip_info=skip_info,
        )
        results[check] = {}
        if result['result'] == CheckResult.SKIPPED:
            results[check]['result'] = result['result']
            results[check]['suppress_comment'] = result['suppress_comment']
            results[check]['results_configuration'] = None
        else:
            results[check]['result'] = cast("CheckResult", result['result'][0])
            results[check]['results_configuration'] = cast("dict[str, Any]", result['result'][1])



from __future__ import annotations

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck

from typing import Any


class ACREnableZoneRedundancy(BaseResourceCheck):

    def __init__(self) -> None:
        """
        Zone redundancy provides resiliency and high availability to
        a registry or replication resource in a specific region. Supported on Premium.
        """
        name = "Ensure Azure Container Registry (ACR) is zone redundant"
        id = "CKV_AZURE_233"
        supported_resources = ("azurerm_container_registry",)
        categories = (CheckCategories.BACKUP_AND_RECOVERY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        # check registry. default=false
        if conf.get("zone_redundancy_enabled", []) != [True]:
            return CheckResult.FAILED

        # check each replica. default=false
        replications = conf.get("georeplications", {})
        for replica in replications:
            zone_redundancy_enabled = replica.get('zone_redundancy_enabled', [])
            if zone_redundancy_enabled != [True]:
                return CheckResult.FAILED

        return CheckResult.PASSED


check = ACREnableZoneRedundancy()



from typing import Any, List

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class MSKClusterEncryption(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure MSK Cluster encryption in rest and transit is enabled"
        id = "CKV_AWS_81"
        supported_resources = ['AWS::MSK::Cluster']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> Any:
        # Note: As long as the 'EncryptionInfo' block is specified, the cluster
        # will be encrypted at rest even if 'DataVolumeKMSKeyId' is not specified
        if 'Properties' in conf.keys():
            if 'EncryptionInfo' in conf['Properties'].keys():
                encryption = conf['Properties']['EncryptionInfo']
                if 'EncryptionInTransit' in encryption:
                    transit = encryption['EncryptionInTransit']
                    if 'ClientBroker' in transit and transit['ClientBroker'] != 'TLS' or \
                            'InCluster' in transit and transit['InCluster'] is False:
                        return CheckResult.FAILED
                return CheckResult.PASSED
        return CheckResult.FAILED

    def get_evaluated_keys(self) -> List[str]:
        return ['Properties/EncryptionInfo/EncryptionInTransit/ClientBroker',
                'Properties/EncryptionInfo/EncryptionInTransit/InCluster']


check = MSKClusterEncryption()



from typing import List, Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class DeprecatedLambdaRuntime(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure Lambda Runtime is not deprecated"
        id = "CKV_AWS_363"
        supported_resources = ['aws_lambda_function']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "runtime"

    def get_forbidden_values(self) -> List[Any]:
        # Source: https://docs.aws.amazon.com/lambda/latest/dg/lambda-runtimes.html
        return ["dotnetcore3.1", "nodejs12.x", "python3.6", "python2.7", "dotnet5.0", "dotnetcore2.1", "ruby2.5",
                "nodejs10.x", "nodejs8.10", "nodejs4.3", "nodejs6.10", "dotnetcore1.0", "dotnetcore2.0",
                "nodejs4.3-edge", "nodejs", "java8", "python3.7", "go1.x", "provided", "ruby2.7", "nodejs14.x"
                # "nodejs16.x", # Uncomment on Jun 12, 2024
                # "python3.8". # Uncomment on Oct 14, 2024
                # "dotnet7", # Uncomment on May 14, 2024
                # "dotnet6", # Uncomment on Nov 12, 2024
                ]


check = DeprecatedLambdaRuntime()



import re

seconds_per_unit = {"s": 1, "m": 60, "h": 3600, "d": 86400, "w": 604800}
seconds_per_unit_regex = r"^\d+[s|m|h|d|w]"


# method 'str.removeprefix()' was added in Python 3.9
def removeprefix(input_str: str, prefix: str) -> str:
    if input_str.startswith(prefix):
        return input_str[len(prefix):]
    return input_str


# in case of comparing paths from the BE and from the client, we have to make sure the structures are the same
# e.g: in windows the seperator for the path is '\' while in linux/max it is '/'
def align_path(path: str) -> str:
    return path.replace('\\', '/')


def convert_to_seconds(input_str: str) -> int:
    if re.search(seconds_per_unit_regex, input_str) is None:
        raise Exception(f"format error for input str, usage: {seconds_per_unit_regex}")
    return int(input_str[:-1]) * seconds_per_unit[input_str[-1]]



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class CognitiveServicesEnableLocalAuth(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that Cognitive Services accounts enable local authentication"
        id = "CKV_AZURE_236"
        supported_resources = ('azurerm_cognitive_account', )
        categories = (CheckCategories.NETWORKING, )
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def get_inspected_key(self):
        return 'local_auth_enabled'

    def get_expected_value(self):
        return False


check = CognitiveServicesEnableLocalAuth()



from typing import Dict, Union, List, Optional

from checkov.common.output.report import Report
from checkov.common.sast.consts import POLICIES_ERRORS, POLICIES_ERRORS_COUNT, SOURCE_FILES_COUNT, POLICY_COUNT, SastLanguages
from checkov.common.sast.report_types import PrismaReport


class CDKReport(Report):
    def __init__(self, check_type: str, metadata: Dict[str, Optional[Union[str, int, List[str]]]], language: SastLanguages, cdk_report: PrismaReport):
        super().__init__(check_type)
        self.metadata = metadata
        self.language: SastLanguages = language
        self.cdk_report: PrismaReport = cdk_report
        # In case we dont have sast report for this lang
        self.empty_sast_report: PrismaReport = PrismaReport(rule_match={language: {}}, profiler={}, errors={}, run_metadata={}, imports={}, reachability_report={}, skipped_checks_by_file={})

    def get_summary(self) -> Dict[str, Union[int, str]]:
        base_summary: Dict[str, Union[int, str]] = super().get_summary()

        err_str = ""
        policies_errors_count = 0
        policies_errors = self.metadata.get(POLICIES_ERRORS)
        if isinstance(policies_errors, list) and policies_errors:
            policies_errors_count = len(policies_errors)
            for e in policies_errors:
                err_str += f"\t- {e}\n"
        base_summary[POLICIES_ERRORS] = err_str
        base_summary[POLICIES_ERRORS_COUNT] = policies_errors_count
        source_files_count = self.metadata.get(SOURCE_FILES_COUNT)
        if isinstance(source_files_count, int) or isinstance(source_files_count, str):
            base_summary[SOURCE_FILES_COUNT] = str(source_files_count)

        policy_count = self.metadata.get(POLICY_COUNT)
        if isinstance(policy_count, int) or isinstance(policy_count, str):
            base_summary[POLICY_COUNT] = policy_count

        return base_summary
